package com.netease.nim.uikit.business.contact.core.util

import android.content.Context
import android.view.View
import android.widget.ListView
import android.widget.Toast
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.api.model.session.SessionCustomization
import com.netease.nim.uikit.business.contact.core.item.AbsContactItem
import com.netease.nim.uikit.business.contact.core.item.ContactTypeHelper
import com.netease.nim.uikit.business.recent.RecentDraftHelper
import com.netease.nim.uikit.business.reply.MsgServiceUtil
import com.netease.nim.uikit.business.reply.PhoneContactMsgService
import com.netease.nim.uikit.business.session.actions.*
import com.netease.nim.uikit.business.session.module.ModuleProxy
import com.netease.nim.uikit.business.session.module.input.IInputPanel
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.common.util.sys.ScreenUtil
import com.bullet.libcommonutil.util.UIHandler.getHandler
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.netease.nimlib.sdk.RequestCallback
import com.netease.nimlib.sdk.msg.MessageBuilder
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import com.netease.nimlib.sdk.msg.model.IMMessage
import java.util.*

/**
 * 文本快捷回复的配置类
 * 继承 ModuleProxy
 */
class InputModuleProxy(private val inputPanel: IInputPanel,
                       val listView: ListView,
                       val context: Context) : ModuleProxy {

    val draftHelper = RecentDraftHelper()

    override fun sendMessage(msg: IMMessage?): Boolean {

        msg ?: return false

        if (NimUIKit.getContactProvider().isPhoneContactByRawId(msg.sessionId)) {
//            createSession(msg.sessionId, msg)
            PhoneContactMsgService(msg,false).sendMessage(msg.sessionId)
        } else {
            sendMsg(msg)
        }

        return true
    }

    /**
     * 通讯录好友快捷回复时，要先创建会话
     */
    private fun createSession(rawContactId: String, msg: IMMessage) {
        val numbers = NimUIKit.getContactProvider().getNumberByContactId(rawContactId)
        val name = NimUIKit.getContactProvider().getNameByContactId(rawContactId)
        PhoneContactSessionCreator.create(context, numbers[0], name) {
            val message = MessageBuilder.createTextMessage(it.userProfile?.accid, msg.sessionType, msg.content)
            sendMsg(message)
        }
    }

    private fun sendMsg(msg: IMMessage) {
        val result = MsgServiceUtil.sendMessage(msg, null, object : RequestCallback<Any> {
            override fun onSuccess(p0: Any?) {
                NimToast.makeToast(context, R.string.send_success, Toast.LENGTH_SHORT).show()
            }

            override fun onFailed(p0: Int) {
                NimToast.makeToast(context, R.string.send_fail, Toast.LENGTH_SHORT).show()
            }

            override fun onException(p0: Throwable?) {
            }

        })

        if (result) {
            inputPanel.hideSelf()
        }
    }

    override fun onInputPanelExpand() {
    }

    override fun onEmoticonPanelCollapse() {
    }

    override fun collapseInputPanel(): Boolean {
        return inputPanel.collapse(false)
    }

    override fun isLongClickEnabled(): Boolean {
        return false
    }

    override fun markSingleReply(replyMsg: IMMessage?, replyName: String?) {
        inputPanel.setReplyMsg(replyMsg, replyName)
    }

    override fun onAvatarLongClicked(item: IMMessage?) {
        inputPanel.onAvatarLongClicked(item)
    }

    override fun revokeMessage(origMsg: IMMessage?) {
        inputPanel.reuseMsg(origMsg)
    }

    override fun onSwitchQuickReply(mVoiceQuickReply: Boolean) {
    }


    fun getActionList(customization: SessionCustomization?, contactItem: AbsContactItem): List<BaseAction> {
        val actions = ArrayList<BaseAction>()
        actions.add(AlbumOrVideoAction())
        actions.add(TakeVideoOrImageAction())
        actions.add(LocationAction())

        if (customization?.actions != null) {
            actions.addAll(customization.actions)
            // 手机联系人过滤掉红包
            if (ContactTypeHelper.isPhoneContact(contactItem)) {
                val iterator = actions.iterator()

                while (iterator.hasNext()) {
                    if (iterator.next() is RedAction) {
                        iterator.remove()
                        break
                    }
                }
            }
        }
        return actions
    }

    fun getSessionCustomization(sessionTypeEnum: SessionTypeEnum): SessionCustomization {
        return if (sessionTypeEnum == SessionTypeEnum.Team)
            NimUIKitImpl.getCommonTeamSessionCustomization()
        else
            NimUIKitImpl.getCommonP2PSessionCustomization()
    }

    fun scrollRecycleView(touchView: View) {
        val touchViewY = ScreenUtil.getLocationOnScreen(touchView)[1] + touchView.measuredHeight
        getHandler().postDelayed({ scrollRecycleView(touchViewY, inputPanel.inputPanelLocation) }, 300)
    }

    private fun scrollRecycleView(touchViewY: Int, inputPanelY: Int) {
        if (inputPanelY in 1..(touchViewY - 1)) {
            listView.smoothScrollBy(touchViewY - inputPanelY, 100)
        }
    }

}

enum class InputType {
    Contact, Session
}