package com.netease.nim.uikit.business.contact.core.util

import android.content.Context
import android.text.TextUtils
import android.widget.Toast
import com.bullet.chat.grpc.GetProfileRequest
import com.bullet.chat.grpc.InviteSignupRequest
import com.bullet.chat.grpc.UserProfile
import com.netease.nim.uikit.api.NimUIKit
import com.smartisan.libstyle.toast.NimToast
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.ServerCodeErrorUtil
import smartisan.cloud.im.framwork.NimSingleThreadExecutor

object PhoneContactSessionCreator {
    private val TAG = this@PhoneContactSessionCreator::class.java.name

    fun create(context: Context, number: String, name: String, callback: (CreatorResult) -> Unit) {
        NimSingleThreadExecutor.getInstance().execute(object : NimSingleThreadExecutor.NimTask<CreatorResult> {
            override fun runInBackground(): CreatorResult {
                val result = CreatorResult(0, "", null)
                first(number, result = result)
                if (result.errorCode == 63) {
                    secound(number, name, result)
                }

                return result
            }

            override fun onCompleted(result: CreatorResult) {
                if (result.isError()) {
                    val code = result.errorCode
                    when (code) {
                        8 -> NimToast.makeToast(context, "请求过于频繁，请稍后再试", Toast.LENGTH_SHORT).show()
                        61 -> NimToast.makeToast(context, "手机号码无效", Toast.LENGTH_SHORT).show()
                        66 -> NimToast.makeToast(context, "请求超时，请稍后再试", Toast.LENGTH_SHORT).show()
                        else -> NimToast.makeToast(context, "邀请注册失败", Toast.LENGTH_SHORT).show()
                    }
                } else {
                    NimUIKit.getContactProvider().setPhoneContact(result.userProfile?.accid, number)
                }

                callback.invoke(result)
            }

        })
    }

    fun first(phone: String, account: String? = null, lmId: String? = null, result: CreatorResult) {
        try {
            val stub = ProtoBuffManager.getInstance().doGetAccountStub()
            val builder = GetProfileRequest.newBuilder()
            if (!TextUtils.isEmpty(phone)) {
                builder.cellphone = phone
            }
            if (!TextUtils.isEmpty(account)) {
                builder.accid = account
            }
            if (!TextUtils.isEmpty(lmId)) {
                builder.lmId = lmId
            }
            result.userProfile = stub.getProfile(builder.build())
        } catch (e: Exception) {
            result.errorCode = ServerCodeErrorUtil.getErrorCode(e)
        }

    }

    fun secound(phone: String, name: String, result: CreatorResult) {
        try {
            val stub = ProtoBuffManager.getInstance().doGetAccountStub()
            result.userProfile = stub.inviteSignup(
                    InviteSignupRequest.newBuilder()
                            .setInviteeCellphone(phone)
                            .setInviteeNickname(name)
                            .build())
        } catch (e: Exception) {
            result.errorMsg = ServerCodeErrorUtil.getErrorDesc(e)
        }

    }

    data class CreatorResult(var errorCode: Int = 0,
                             var errorMsg: String = "",
                             var userProfile: UserProfile?) {
        fun isError() = errorCode != 0 || errorMsg.isNotEmpty()
    }

}