package com.netease.nim.uikit.business.forward

import android.content.Context
import com.netease.nim.uikit.business.session.extension.ForwardMsgRemarkAttachment
import com.netease.nim.uikit.business.session.extension.SingleReplyAttachment
import com.netease.nim.uikit.business.session.helper.AudioMessageHelper
import com.netease.nim.uikit.common.server.IMMsgHelper
import com.netease.nim.uikit.common.server.IMMsgHelper.IS_CHECK_KEY
import com.netease.nimlib.sdk.avchat.model.AVChatAttachment
import com.netease.nimlib.sdk.msg.attachment.AudioAttachment
import com.netease.nimlib.sdk.msg.attachment.NotificationAttachment
import com.netease.nimlib.sdk.msg.constant.MsgTypeEnum
import com.netease.nimlib.sdk.msg.model.IMMessage

/**
 * 检查消息类型是否支持转发，如果不支持转发，将
 * 集合中所有不支持转发的消息类型列出，并弹出弹
 * 窗提示
 */
object ForwardTypeChecker {

    private val blackListMsgType = mutableMapOf<Class<*>, String>(
            AVChatAttachment::class.java to "通话提示信息",
            AudioAttachment::class.java to "纯语音",
            NotificationAttachment::class.java to "通知",
            ForwardMsgRemarkAttachment::class.java to "多条转发消息",
            SingleReplyAttachment::class.java to "引用回复"
    )

    fun checkForwardMessages(context: Context, messageList: MutableList<IMMessage>): ForwardTypeCheckerResult {

        val tempBlack = mutableMapOf<Class<*>, String>()
        tempBlack.putAll(blackListMsgType)

        var filterDes = ""
        var canForward = false

        messageList.forEach {

            if (IMMsgHelper.getMsgExtension(it, IS_CHECK_KEY, false)) {
                if (it.msgType == MsgTypeEnum.audio
                        && AudioMessageHelper.getAttachedText(it).isNullOrEmpty()) {
                    filterDes = "$filterDes${tempBlack[AudioAttachment::class.java]}、"
                } else if (it.attachment != null) {

                    val atClazz = it.attachment::class.java
                    if (tempBlack.containsKey(atClazz)) {
                        filterDes = "$filterDes${tempBlack[atClazz]}、"
                        tempBlack.remove(atClazz)
                    } else {
                        //转发的消息中包含黑名单外的消息
                        canForward = true
                    }

                } else {
                    canForward = true
                }

                if (tempBlack.isEmpty()) return@forEach
            }

        }

        if (filterDes.isNotEmpty()) {
            filterDes = filterDes.substring(0, filterDes.length - 1)
        }

        return ForwardTypeCheckerResult(canForward, filterDes.isNotEmpty(), filterDes)
    }

    data class ForwardTypeCheckerResult(private val canForward: Boolean,
                                        private val showConfirm: Boolean,
                                        val des: String?) {
        fun canForward() = canForward

        fun showConfirm() = showConfirm
    }
}