package com.netease.nim.uikit.business.forward

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.widget.ImageView
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.forward.adapter.ForwardMsgAdapter
import com.netease.nim.uikit.business.forward.data.NIMFileObject
import com.netease.nim.uikit.business.forward.data.NIMImageObject
import com.netease.nim.uikit.business.forward.data.NIMLocationObject
import com.netease.nim.uikit.business.forward.data.NIMVideoObject
import com.netease.nim.uikit.business.forward.presenter.ForwardMsgPresenter
import com.netease.nim.uikit.business.session.constant.Extras
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.ui.dialog.SingleMediaWatchDialog
import com.netease.nim.uikit.impl.NimUIKitImpl
import kotlinx.android.synthetic.main.activity_multi_forward_msg_layout.*
import com.bullet.libcommonutil.log.Logger

class MultiForwardMessageActivity : UI(), ForwardMsgPresenter.ForwardMsgView {

    companion object {

        private val TAG = MultiForwardMessageActivity::class.java.name

        fun start(context: Context, uuid: String) {
            val starter = Intent(context, MultiForwardMessageActivity::class.java)
            starter.putExtra(Extras.EXTRA_IM_MESSAGE, uuid)
            context.startActivity(starter)
        }
    }

    private val forwardMsgPresenter by lazy {
        ForwardMsgPresenter(this)
    }

    private val uuid by lazy {
        intent.getStringExtra(Extras.EXTRA_IM_MESSAGE)
    }

    private val forwardMsgAdapter = ForwardMsgAdapter()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_multi_forward_msg_layout)
        forwardMsgPresenter.loadData(uuid)

        msgRecyclerView.layoutManager = LinearLayoutManager(this)
        msgRecyclerView.adapter = forwardMsgAdapter


        forwardMsgAdapter.viewHolderConfig.subviewOnClickListenerMap.apply {
            put(R.id.msgImageView, { model, view ->

                val singleMediaWatchDialog = SingleMediaWatchDialog(this@MultiForwardMessageActivity);
                when (model) {
                    is NIMImageObject -> {
                        singleMediaWatchDialog
                                .setThumbUrl(model.thumbUrl)
                                .setImageUrl(model.url)
                    }
                    is NIMVideoObject -> {
                        singleMediaWatchDialog
                                .setThumbUrl(model.coverUrl)
                                .setVideoUrl(model.url)
                                .setVideoSize(model.width, model.height)
                                .setVideoDuration(model.duration)
                    }
                    else -> {
                        Logger.i(TAG, "image view perform click but message is not image")
                        return@put
                    }
                }

                val targetView: ImageView = view.findViewById(R.id.msgImageView)
                singleMediaWatchDialog.show(targetView)
            })

            put(R.id.coverImageView, { model, _ ->
                if (model !is NIMVideoObject) {
                    Logger.i(TAG, "view view perform click but message is not video")
                    return@put
                }
            })

            put(R.id.card_root_layout, { model, _ ->
                when (model) {
                    is NIMLocationObject -> {
                        NimUIKitImpl.getLocationProvider().openMap(this@MultiForwardMessageActivity, model.longitude, model.latitude, model.title)
                    }

                    is NIMFileObject -> {
                        SimpleFileDownloadActivity.start(this@MultiForwardMessageActivity, model.displayName, model.url, model.extension)
                    }
                }
            })
        }

    }

    override fun dataLoaded(data: MutableList<out Any>) {
        forwardMsgAdapter.addItems(data, true)
    }

    override fun loadDataFailed() {
    }


}
