package com.netease.nim.uikit.business.forward

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.widget.Toast
import com.netease.nim.uikit.R
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.rx.DisposableCollector
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.common.util.file.FileTypeUtils
import smartisan.cloud.im.util.FileUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import com.bullet.libcommonutil.log.Logger
import zlc.season.rxdownload3.RxDownload
import zlc.season.rxdownload3.core.*
import java.io.File
import kotlinx.android.synthetic.main.file_download_activity.*

class SimpleFileDownloadActivity : UI() {

    companion object {
        private val TAG = "SimpleFileDownloadActivity==="
        private const val DISPLAY_NAME = "DISPLAY_NAME"
        private const val FILE_URL = "FILE_URL"
        private const val FILE_EXTENSION = "FILE_EXTENSION"
        fun start(context: Context, display: String, fileUrl: String, extension: String) {
            val intent = Intent()
            intent.setClass(context, SimpleFileDownloadActivity::class.java)
            intent.putExtra(DISPLAY_NAME, display)
            intent.putExtra(FILE_URL, fileUrl)
            intent.putExtra(FILE_EXTENSION, extension)
            context.startActivity(intent)
        }
    }

    private val displayName: String by lazy {
        intent.getStringExtra(DISPLAY_NAME)
    }

    private val fileUrl: String by lazy {
        intent.getStringExtra(FILE_URL)
    }
    private val fileExtension: String by lazy {
        intent.getStringExtra(FILE_EXTENSION)
    }

    private val mission by lazy {
        Mission(fileUrl, "${System.currentTimeMillis()}.$fileExtension", CACHE_PATH)
    }

    private val startDownloadClick by lazy {
        View.OnClickListener {
            RxDownload.start(mission).subscribe()
        }
    }

    private val pauseClick by lazy {
        View.OnClickListener {
            RxDownload.stop(mission).subscribe()
        }
    }

    private val CACHE_PATH = RxDownloader.DOWNLOADER_CACHE

    private val dc = DisposableCollector()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.file_download_activity)

        FileUtil.disableFileUriExposure()

        file_name.text = displayName
        file_type.setImageResource(FileTypeUtils.getFileIconByName(displayName))
        download_btn.text = ""
        download_btn.isEnabled = false


        dc.collect {
            RxDownload.create(mission, false)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({

                        when (it) {
                            is Normal -> {
                                changeBtn(getString(R.string.download_text), startDownloadClick)
                            }
                            is Succeed -> {
                                updateUIToDownloaded()
                            }

                            is Suspend -> {
                                changeBtn(getString(R.string.message_attachment_transfer_state_pause,
                                        FileUtil.formatFileSize(it.downloadSize), FileUtil.formatFileSize(it.totalSize)), startDownloadClick)
                            }
                            is Failed -> {

                            }
                            is Downloading -> {
//                                this.downloadSize = it.downloadSize
//                                this.totalSize = it.totalSize

                                changeBtn(getString(R.string.message_attachment_transfer_state_canche,
                                        FileUtil.formatFileSize(it.downloadSize), FileUtil.formatFileSize(it.totalSize)), pauseClick)
                            }
                        }

                    }, {
                        Logger.e(TAG, "create error: $it")
                    })
        }

    }

    private fun changeBtn(text: String, clickListener: View.OnClickListener) {
        download_btn.isEnabled = true
        download_btn.text = text
        download_btn.setOnClickListener(clickListener)
    }


    private fun updateUIToDownloaded() {
        download_btn.isEnabled = true
        dc.collect {
            RxDownload.file(mission)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({
                        preViewFile(it.path, it.extension)
                    })
        }
    }

    private fun preViewFile(path: String, extension: String) {
        val mimeType = FileTypeUtils.getFileMimeTypeBySuffix(extension)
        if (isPreViewable(extension)) return
        var intent: Intent? = makeIntent(mimeType, path)
        if (intent!!.resolveActivity(packageManager) == null) {
            intent = makeDefaultIntent(path, mimeType, extension)
            if (intent == null || intent.resolveActivity(packageManager) == null) {
                NimToast.makeToast(this@SimpleFileDownloadActivity, R.string.message_attachment_transfer_cannot_open, Toast.LENGTH_SHORT).show()
                return
            }
        }
        startActivity(intent)
    }

    private fun makeIntent(mimeType: String, path: String): Intent {
        val intent = Intent(Intent.ACTION_VIEW)
        intent.addCategory("android.intent.category.DEFAULT")
        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.N) {
            //            File file = (new File(path));
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
            intent.setDataAndType(Uri.fromFile(File(path)), mimeType)
        } else {
            intent.setDataAndType(Uri.fromFile(File(path)), mimeType)
        }
        return intent
    }

    private fun setDataAndType(intent: Intent, mimeType: String, path: String, ext: String): Intent {
        val extension = FileUtil.getExtensionName(path)
        if (TextUtils.isEmpty(extension)) {
            intent.setDataAndType(Uri.fromFile(File("$path.$ext")), mimeType)
        } else {
            intent.setDataAndType(Uri.fromFile(File(path)), mimeType)
        }
        return intent
    }

    private fun makeDefaultIntent(path: String, mimeType: String, ext: String): Intent? {
        val intent = Intent(Intent.ACTION_VIEW)
        try {
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            setDataAndType(intent, mimeType, path, ext)
            //grant other Apps the permission to access DocumentUri in usb storage
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        } catch (e: Exception) {
            Logger.w(TAG, "Failed to open $path memeType: $mimeType : $e")
            return null
        }

        return intent
    }

    private fun isPreViewable(extension: String): Boolean {
        val isApk = "apk".equals(extension, ignoreCase = true)
        if (isApk) {
            file_tips.visibility = View.GONE
            download_btn.setText(R.string.message_attachment_apk_install)
        } else {
            file_tips.visibility = View.VISIBLE
            download_btn.setText(R.string.message_attachment_transfer_open_other)
        }
        return false
    }


    override fun onDestroy() {
        super.onDestroy()
        RxDownload.stop(mission).subscribe()
        dc.clearDisposable()
    }

}