package com.netease.nim.uikit.business.forward.presenter

import com.bullet.chat.grpc.ImGrpc
import com.bullet.chat.grpc.TransferMFRequest
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.netease.nim.uikit.business.forward.data.ForwardModel
import com.netease.nim.uikit.business.forward.data.ForwardModelHelper
import com.netease.nim.uikit.common.presenter.HandleDisposablePresenter
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.grpc.ProtoBuffManager

class ForwardMsgPresenter(private val forwardMsgView: ForwardMsgView) : HandleDisposablePresenter() {

    fun loadData(id: String?) {
        if (id.isNullOrEmpty()) {
            return
        }

        val idLong: Long
        try {
            idLong = id!!.toLong()
        } catch (e: ClassCastException) {
            forwardMsgView.loadDataFailed()
            return
        }

        collectDisposable {
            Observable.fromCallable {
                val stub = ProtoBuffManager.getInstance().doGetImStub(false) as ImGrpc.ImBlockingStub
                val builder = TransferMFRequest.newBuilder()
                builder.id = idLong
                stub.transferMF(builder.build())
            }
                    .map {
                        Gson().fromJson<MutableList<ForwardModel>>(it.data, object : TypeToken<MutableList<ForwardModel>>() {}.type)
                    }
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({
                        it.mapNotNull {
                            ForwardModelHelper.getAnyNotNull(it)
                        }.toMutableList().let {
                            forwardMsgView.dataLoaded(it)
                        }

                    }, {
                        forwardMsgView.loadDataFailed()
                    })
        }
    }

    interface ForwardMsgView {
        fun dataLoaded(data: MutableList<out Any>)
        fun loadDataFailed()
    }

}