package com.netease.nim.uikit.business.forward.viewholder

import android.view.View
import com.bullet.messenger.widgetkit.viewholder.ViewHolderFactory
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.forward.data.ForwardMsgModel
import com.netease.nim.uikit.business.forward.data.NIMFileObject
import com.netease.nim.uikit.business.forward.data.NIMLocationObject
import com.netease.nim.uikit.business.location.LocationHelper
import com.netease.nim.uikit.business.session.viewholder.MsgViewHolderLocation.MAP_SHOT_SIZE
import smartisan.cloud.im.util.FileUtil
import com.netease.nim.uikit.impl.NimUIKitImpl

import com.bullet.libcommonutil.log.Logger
import kotlinx.android.synthetic.main.forward_card_message_layout.view.*

class CardMessageViewHolder(itemView: View) : BaseMessageViewHolder<ForwardMsgModel>(itemView) {

    companion object : ViewHolderFactory<CardMessageViewHolder> {

        private val TAG: String = CardMessageViewHolder::class.java.name

        override fun getLayoutId(): Int = R.layout.layout_msg_card

        override fun getViewHolderClass(): Class<*> = CardMessageViewHolder::class.java

    }

    override fun configView(model: ForwardMsgModel) {
        itemView.card_root_layout.setBackgroundResource(R.drawable.nim_recent_message_item_round_bg)
        when (model) {
            is NIMLocationObject -> fillByLocation(model)
            is NIMFileObject -> fillByFile(model)
            else -> Logger.e(TAG, "message type not for CardMessageViewHolder!!!!")
        }
    }

    private fun fillByFile(msg: NIMFileObject) {
        itemView.thumb_image_view.visibility = View.INVISIBLE
        itemView.icon_background.setBackgroundResource(NimUIKitImpl.getOptions().getIconBg(msg.extension))
        itemView.icon_view.setBackgroundResource(NimUIKitImpl.getOptions().getIcon(msg.extension))
        itemView.file_name.text = msg.displayName

        itemView.file_des.text = FileUtil.formatFileSize(msg.fileLength)

        itemView.file_type.text = itemView.context.getString(NimUIKitImpl.getOptions().getFileTypeText(msg.extension))
    }

    private fun fillByLocation(msg: NIMLocationObject) {
        itemView.icon_view.setBackgroundResource(R.drawable.ic_bubbles_ablum_location)
        itemView.file_name.text = msg.title
        itemView.file_des.text = msg.title
        itemView.file_type.text = itemView.context.getString(R.string.address)
        itemView.thumb_image_view.visibility = View.VISIBLE
        itemView.thumb_image_view.loadAsUrl(LocationHelper.getStaticMapUrl(msg.latitude, msg.longitude), MAP_SHOT_SIZE, MAP_SHOT_SIZE, 0)
    }

}