package com.netease.nim.uikit.business.redpacket

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Paint
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.FrameLayout
import android.widget.TextView
import com.netease.nim.uikit.R
import kotlinx.android.synthetic.main.b2c_progress_layout.view.*

class B2CProgressLayout : FrameLayout {

    private val OFFSET = 0.0542f
    private val HALF_OFFSET = 0.0421f

    private val POSITION_1 = 0.1164f
    private val POSITION_2 = 0.2269f
    private val POSITION_3 = 0.4438f
    private val POSITION_4 = 1 - OFFSET

    private val LEVEL_1 = 1f
    private val LEVEL_2 = 5f
    private val LEVEL_3 = 20f
    private val LEVEL_4 = 100f

    private var inited = false
    private var amount: Float = 0.0f

    constructor(context: Context?) : super(context) {
        initView()
    }

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {
        initView()
    }

    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        initView()
    }

    private fun initView() {
        LayoutInflater.from(context).inflate(R.layout.b2c_progress_layout, this, true)

        post {
            val bitmap = Bitmap.createBitmap(measuredWidth, measuredHeight, Bitmap.Config.ARGB_8888)
            val canvas = Canvas(bitmap)
            fakeCoverView.draw(canvas)
            coverView.setImageBitmap(bitmap)

            amountView1.x = measuredWidth * POSITION_1
            amountView2.x = measuredWidth * POSITION_2
            amountView3.x = measuredWidth * POSITION_3

            inited = true

            setAmount(amount)
        }

    }

    fun setProgress(progress: Float) {
        coverView.layoutParams.width = (measuredWidth * progress).toInt()
        requestLayout()
    }

    fun setAmount(amount: String?) {

        if (amount.isNullOrEmpty() || TextUtils.equals(amount, context.getString(R.string.default_money))) {
            return
        }

        setAmount(amount?.toFloatOrNull() ?: 0.00f)
    }

    fun setAmount(realAmount: Float) {

        if (!inited) {
            amount = realAmount
            return
        }

        if (realAmount >= LEVEL_1) {
            changeText(amountView1)
        }

        if (realAmount >= LEVEL_2) {
            changeText(amountView2)
        }

        if (realAmount >= LEVEL_3) {
            changeText(amountView3)
        }

        if (realAmount >= LEVEL_4) {
            changeText(amountView4)
        }

        when {
            realAmount < LEVEL_1 -> {
                var progress = (POSITION_1 - OFFSET) * realAmount + OFFSET
                if (progress > POSITION_1) {
                    progress = POSITION_1
                }
                setProgress(progress)
            }
            realAmount == LEVEL_1 -> setProgress(POSITION_1 + HALF_OFFSET)
            realAmount < LEVEL_2 -> {
                var progress = (POSITION_2 - POSITION_1 - HALF_OFFSET) * ((realAmount - LEVEL_1) / (LEVEL_2 - LEVEL_1)) + POSITION_1 + HALF_OFFSET
                if (progress > POSITION_2) {
                    progress = POSITION_2
                }
                setProgress(progress)
            }
            realAmount == LEVEL_2 -> setProgress(POSITION_2 + HALF_OFFSET)
            realAmount < LEVEL_3 -> {
                var progress = (POSITION_3 - POSITION_2 - HALF_OFFSET) * ((realAmount - LEVEL_2) / (LEVEL_3 - LEVEL_2)) + POSITION_2 + HALF_OFFSET
                if (progress > POSITION_3) {
                    progress = POSITION_3
                }
                setProgress(progress)
            }
            realAmount == LEVEL_3 -> setProgress(POSITION_3 + HALF_OFFSET)
            realAmount < LEVEL_4 -> {
                var progress = (POSITION_4 - POSITION_3 - HALF_OFFSET) * ((realAmount - LEVEL_3) / (LEVEL_4 - LEVEL_3)) + POSITION_3 + HALF_OFFSET
                if (progress > POSITION_4) {
                    progress = POSITION_4
                }
                setProgress(progress)
            }
            realAmount == LEVEL_4 -> setProgress(1f)
        }
    }

    private fun changeText(textView: TextView) {
        textView.apply {
            setTextColor(ContextCompat.getColor(context, R.color.red_packet_progress_text_color))
        }
    }

}