package com.netease.nim.uikit.business.redpacket

import android.content.Context
import android.content.Intent
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import android.widget.Toast
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.contact.core.item.AbsContactItem
import com.netease.nim.uikit.business.contact.core.item.ContactTypeHelper
import com.netease.nim.uikit.business.contact.core.item.ItemTypes
import com.netease.nim.uikit.business.contact.pick.view.IMPickContactView
import com.netease.nim.uikit.business.contact.pick.view.PickContactView
import com.netease.nim.uikit.business.contact.selector.view.ContactSelectView
import com.netease.nim.uikit.common.activity.titlebar.NimTitleBar
import com.smartisan.libstyle.toast.NimToast

class PhoneContactSelectView : ContactSelectView {
    constructor(intent: Intent?, context: Context?) : super(intent, context)
    constructor(context: Context?) : super(context)

    override fun initTitleBar(titleBar: NimTitleBar) {
        super.initTitleBar(titleBar)
        val titleView = titleBar.getTitleView<TextView>()
        titleView.setText(R.string.create_group_session)
    }

    override fun initHeader(header: LinearLayout) {
        super.initHeader(header)
        showHeadLine(false)
    }

    override fun initPickContactView(contactView: IMPickContactView) {
        super.initPickContactView(contactView)
        val params = contactView.layoutParams as RelativeLayout.LayoutParams
        params.setMargins(params.leftMargin, 0, params.rightMargin, params.bottomMargin)

        setConfirmPickListener(PhoneContactConfirmPickListener(this))

        setItemTypes(ItemTypes.PHONE_CONTACT)
    }


    class PhoneContactConfirmPickListener(private val contactSelectView: PhoneContactSelectView) : PickContactView.OnConfirmPickListener {

        override fun onPick(absContactItem: AbsContactItem, checked: Boolean): Boolean {

            if (!checked) {
                return true
            }

            if (ContactTypeHelper.isPhoneContact(absContactItem) && getPhoneContactCountInPick(1) > MAX_PHONE_CONTACT_COUNT) {
                NimToast.makeToast(contactSelectView.context,
                        contactSelectView.context.getString(R.string.max_phone_contact_tips, MAX_PHONE_CONTACT_COUNT),
                        Toast.LENGTH_LONG)
                        .show()
                return false
            }

            if (contactSelectView.pickContactCount + 1 > contactSelectView.maxSelectNum) {
                NimToast.makeToast(contactSelectView.context, contactSelectView.maxSelectedTip, Toast.LENGTH_LONG).show()
                return false
            }

            return true
        }

        private fun getPhoneContactCountInPick(offset: Int): Int {
            var phoneContactCount = offset
            for (contactItem in contactSelectView.imPickContactView.pickContact) {
                if (ContactTypeHelper.isPhoneContact(contactItem)) {
                    phoneContactCount += 1
                }
            }

            return phoneContactCount
        }

        companion object {
            private const val MAX_PHONE_CONTACT_COUNT = 5
        }
    }
}