package com.netease.nim.uikit.business.redpacket

import android.app.Dialog
import android.content.Context
import android.graphics.Rect
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.Window
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import kotlinx.android.synthetic.main.dialog_red_packet.view.*

/**
 * 领取红包对话框
 * Created by lan on 2018/9/6.
 */
class RedPacketDialog private constructor(val builder: Builder) : Dialog(builder.context) {

    private val redPacketLayout by lazy {
        RedPacketLayout(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        setContentView(redPacketLayout)
        window.setBackgroundDrawable(context.resources.getDrawable(android.R.color.transparent))
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
        window.setGravity(Gravity.CENTER)
        window.attributes.dimAmount = 0f

        initView(builder)

    }

    fun initView(builder: Builder?) {
        redPacketLayout.apply {
            stopAnimation()

            flashView.visibility = View.INVISIBLE
            flashView2.visibility = View.INVISIBLE
            fromAccount = builder?.fromAccount
            message = builder?.message
            redPacketStatus = builder?.redPacketStatus
            tip = builder?.tip
            isShowGoToDetail = builder?.isShowGoToDetail

            inviteUrl.setOnClickListener { builder?.goToDetail?.invoke() }
            open.setOnClickListener { builder?.open?.invoke() }
            rootLayout.setOnClickListener { this@RedPacketDialog.dismiss() }
        }
    }

    fun loading(){
        if (!builder.isExpired && !builder.isFinish) {
            redPacketLayout.loading()
        }
    }

    fun opening(avatarRect: Rect){
        redPacketLayout.opening(avatarRect)
    }


    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        redPacketLayout.stopAnimation()
    }

    class Builder(val context: Context) {

        var fromAccount: String? = null
        var message: String = context.getString(R.string.red_packet_msg_default)
        var open: (() -> Unit)? = null
        var goToDetail: (() -> Unit)? = null
        var redPacketStatus: RedPacketStatus? = null
        var type: RedPacketAttachment.RedPacketType? = null

        var tip: String = context.getString(R.string.send_red_packet_tip)
            private set
            get() = when (type) {
                RedPacketAttachment.RedPacketType.TeamAVG -> {
                    context.getString(R.string.send_avg_red_packet_tip)
                }

                RedPacketAttachment.RedPacketType.TeamLucky -> {
                    context.getString(R.string.send_lucky_red_packet_tip)
                }

                else -> context.getString(R.string.send_red_packet_tip)
            }

        // 自己发的红包 or 红包过期 or 红包被领完 可以查看红包详情
        var isShowGoToDetail: Boolean = false
            private set
            get() = TextUtils.equals(fromAccount, NimUIKit.getAccount())
                    || redPacketStatus == RedPacketStatus.FINISH
                    || redPacketStatus == RedPacketStatus.EXPIRED

        var isFinish: Boolean = false
            private set
            get() = redPacketStatus == RedPacketStatus.FINISH

        var isExpired: Boolean = false
            private set
            get() = redPacketStatus == RedPacketStatus.EXPIRED

        fun build(): RedPacketDialog {
            return RedPacketDialog(this)
        }

    }

}
