package com.netease.nim.uikit.business.redpacket

import android.animation.Animator
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.animation.TimeInterpolator
import android.content.Context
import android.graphics.Rect
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.text.TextUtils
import android.text.format.DateUtils
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.animation.*
import android.widget.FrameLayout
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.session.adapter.Anim
import com.netease.nim.uikit.business.uinfo.UserInfoHelper
import com.netease.nim.uikit.common.util.sys.ScreenUtil
import kotlinx.android.synthetic.main.dialog_red_packet.*
import kotlinx.android.synthetic.main.dialog_red_packet.view.*

class RedPacketLayout : FrameLayout {

    var fromAccount: String? = null
        set(value) {
            field = value
            avatar.loadBuddyAvatar(field)
            name.text = getSenderName(field)
        }

    var message: String? = null
        set(value) {
            field = value
            content.text = field
        }

    var redPacketStatus: RedPacketStatus? = null
        set(value) {
            field = value
            when (field) {
                RedPacketStatus.EXPIRED -> {
                    open.visibility = View.INVISIBLE
                    expiredTv.visibility = View.VISIBLE
                    getDoneTv.visibility = View.GONE
                    content.visibility = View.GONE
                }
                RedPacketStatus.FINISH -> {
                    open.visibility = View.INVISIBLE
                    expiredTv.visibility = View.GONE
                    getDoneTv.visibility = View.VISIBLE
                    content.visibility = View.GONE
                }
                else -> {
                    getDoneTv.visibility = View.GONE
                    expiredTv.visibility = View.GONE
                    content.visibility = View.VISIBLE
                }
            }
        }

    var tip: String? = null
        set(value) {
            field = value
            tipView.text = field
        }

    var isShowGoToDetail: Boolean? = false
        set(value) {
            field = value
            inviteUrl.visibility = if (field == true) View.VISIBLE else View.INVISIBLE
        }


    private var iconAnimation: AnimationDrawable? = null
    private var animatorSet: AnimatorSet? = null

    constructor(context: Context?) : super(context) {
        initView()
    }

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs) {
        initView()
    }

    constructor(context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        initView()
    }


    fun initView() {
        LayoutInflater.from(context).inflate(R.layout.dialog_red_packet, this, true)
        // 防止通过 Accessibility Service 自动开红包
        open.setAccessibilityDelegate(object : View.AccessibilityDelegate() {
            override fun performAccessibilityAction(host: View?, action: Int, args: Bundle?): Boolean {
                return true
            }
        })
    }

    fun opening(avatarRect: Rect) {
        animatorSet?.cancel()
        iconAnimation?.stop()
        iconAnimation?.getFrame(0)
        open.setImageDrawable(iconAnimation?.getFrame(0))

        val avatarTransY = avatarRect.top.toFloat() - ScreenUtil.getStatusBarHeight(context)
        val topCoverTransY = topCoverView.y - TopCoverTranslationY
        val bottomCoverTransY = bottomCoverView.y + BottomCoverTranslationY

        AnimatorSet().apply {
            playTogether(getAlphaAnim(tipView, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))
            playTogether(getAlphaAnim(name, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))
            playTogether(getAlphaAnim(expiredTv, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))
            playTogether(getAlphaAnim(getDoneTv, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))
            playTogether(getAlphaAnim(content, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))
            playTogether(getAlphaAnim(inviteUrl, TextAlphaStart, TextAlphaEnd, TextAlphaDuration))

            playTogether(getAlphaAnim(avatarLayout, AvatarAlphaStart, AvatarAlphaEnd, AvatarAlphaDuration))
            playTogether(getTransYAnim(avatarLayout, avatarLayout.y, avatarTransY, AvatarTranslationYDuration, DecelerateInterpolator()))
            playTogether(getScaleAnim(avatarLayout, AvatarScaleStart, AvatarScaleEnd, AvatarScaleDuration, DecelerateInterpolator()))

            playTogether(getScaleAnim(open, CoinScaleStart, CoinScaleEnd, CoinScaleDuration, AccelerateDecelerateInterpolator()))
            playTogether(getAlphaAnim(open, CoinAlphaStart, CoinAlphaEnd, CoinAlphaDuration))

            playTogether(getScaleAnim(flashView, OpenFlashScaleStart, OpenFlashScaleEnd, OpenFlashScaleDuration, AccelerateDecelerateInterpolator()))
            playTogether(getAlphaAnim(flashView, OpenFlashAlphaStart, OpenFlashAlphaEnd, OpenFlashAlphaDuration))

            playTogether(getScaleAnim(flashView2, OpenFlash2ScaleStart, OpenFlash2ScaleEnd, OpenFlash2ScaleDuration, AccelerateDecelerateInterpolator()))
            playTogether(getAlphaAnim(flashView2, OpenFlash2AlphaStart, OpenFlash2AlphaEnd, OpenFlash2AlphaDuration))

            playTogether(getTransYAnim(topCoverView, topCoverView.y, topCoverTransY, TopCoverTranslationYDuration, AccelerateInterpolator()))
            playTogether(getScaleAnim(topCoverView, TopCoverScaleStart, TopCoverScaleEnd, TopCoverScaleDuration, AccelerateDecelerateInterpolator()))
            playTogether(getAlphaAnim(topCoverView, TopCoverAlphaStart, TopCoverAlphaEnd, TopCoverAlphaDuration))

            playTogether(getTransYAnim(bottomCoverView, bottomCoverView.y, bottomCoverTransY, BottomCoverTranslationYDuration, AccelerateInterpolator()))
            playTogether(getScaleAnim(bottomCoverView, BottomCoverScaleStart, BottomCoverScaleEnd, BottomCoverScaleDuration, AccelerateDecelerateInterpolator()))
            playTogether(getAlphaAnim(bottomCoverView, BottomCoverAlphaStart, BottomCoverAlphaEnd, BottomCoverAlphaDuration))

            playTogether(getAlphaAnim(bgColorView, BgAlphaStart, BgAlphaEnd, BgAlphaDuration))
        }.start()

    }

    private fun getScaleAnim(view: View, sv: Float, ev: Float, duration: Long, interpolator: Interpolator = LinearInterpolator(), repeatCount: Int = 0): MutableList<Animator> {
        val a1 = ObjectAnimator.ofFloat(view, View.SCALE_X, sv, ev).apply {
            this.duration = duration
            this.interpolator = interpolator
            this.repeatCount = repeatCount
        }

        val a2 = ObjectAnimator.ofFloat(view, View.SCALE_Y, sv, ev).apply {
            this.duration = duration
            this.interpolator = interpolator
            this.repeatCount = repeatCount
        }

        return mutableListOf(a1, a2)
    }

    private fun getAlphaAnim(view: View, sv: Float, ev: Float, duration: Long, interpolator: Interpolator = LinearInterpolator(), repeatCount: Int = 0): Animator {
        return ObjectAnimator.ofFloat(view, View.ALPHA, sv, ev).apply {
            this.duration = duration
            this.interpolator = interpolator
            this.repeatCount = repeatCount
        }
    }

    private fun getTransYAnim(view: View, sv: Float, ev: Float, duration: Long, interpolator: Interpolator = LinearInterpolator(), repeatCount: Int = 0): Animator {
        return ObjectAnimator.ofFloat(view, View.Y, sv, ev).apply {
            this.duration = duration
            this.interpolator = interpolator
            this.repeatCount = repeatCount
        }
    }

    fun loading() {
        iconAnimation = (open.drawable as AnimationDrawable)
        iconAnimation?.start()

        flashView.visibility = View.VISIBLE
        flashView2.visibility = View.VISIBLE

        animatorSet = AnimatorSet().apply {
            playTogether(getScaleAnim(flashView, FlashScaleStart, FlashScaleEnd, FlashScaleDuration, AccelerateDecelerateInterpolator(), FlashAnimRepeatCount))
            playTogether(getAlphaAnim(flashView, FlashAlphaStart, FlashAlphaEnd, FlashAlphaDuration, repeatCount = FlashAnimRepeatCount))

            playTogether(getScaleAnim(flashView2, Flash2ScaleStart, Flash2ScaleEnd, Flash2ScaleDuration, AccelerateDecelerateInterpolator(), FlashAnimRepeatCount))
            playTogether(getAlphaAnim(flashView2, Flash2AlphaStart, Flash2AlphaEnd, Flash2AlphaDuration, repeatCount = FlashAnimRepeatCount))

        }
        animatorSet?.start()

    }

    fun stopAnimation() {
        iconAnimation?.stop()
        animatorSet?.cancel()
    }

    private fun getSenderName(senderId: String?): String {
        senderId ?: return context.getString(R.string.app_name)
        val currentUserName = UserInfoHelper.getUserDisplayNameWithNewRule(senderId)
        return currentUserName ?: context.getString(R.string.your_friend)
    }
}