package com.netease.nim.uikit.business.redpacket.activity

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.SparseArray
import android.util.SparseIntArray
import android.view.View
import android.view.ViewGroup
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.fragment.C2CRedPacketListFragment
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.activity.titlebar.*
import kotlinx.android.synthetic.main.nim_advanced_team_file_activity.*
import java.util.*

/**
 * C2C红包收支明细
 * Created by lan on 2018/9/17.
 */
class C2CRedPacketListActivity : UI(), View.OnClickListener {

    private var tabLayout : ViewGroup? = null
    private val tabHost = SparseArray<View>()
    private val mTabDataHost = ArrayList<SparseIntArray>()
    private val mReceivedList = SparseIntArray()
    private val mSentList = SparseIntArray()
    private var mCurrentTabIndex = TAB_RECEIVED

    private val receivedFragment by lazy {
        C2CRedPacketListFragment.newInstance(TAB_RECEIVED).apply {
            containerId = R.id.container
        }
    }

    private val sentFragment by lazy {
        C2CRedPacketListFragment.newInstance(TAB_SENT).apply {
            containerId = R.id.container
        }
    }

    companion object {
        val TAB_RECEIVED = 0
        val TAB_SENT = 1

        fun start(context: Context?) {
            context?.startActivity(Intent(context, C2CRedPacketListActivity::class.java))
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_c2c_red_packet_list)

        initTitleBar()
        switchContent(receivedFragment)
    }

    private fun initTitleBar() {
        mTabDataHost.clear()

        mReceivedList.put(TitleBarOptions.TabHostOption.KEY_ID, R.id.received_list);
        mReceivedList.put(TitleBarOptions.TabHostOption.KEY_TAB_TEXT_RES, R.string.received_packet_list);
        mSentList.put(TitleBarOptions.TabHostOption.KEY_ID, R.id.sent_list);
        mSentList.put(TitleBarOptions.TabHostOption.KEY_TAB_TEXT_RES, R.string.sent_packet_list);

        mTabDataHost.add(mReceivedList)
        mTabDataHost.add(mSentList)
        findViewById<NimTitleBar>(R.id.toolbar).setBackgroundResource(R.drawable.titlebar_red_bg)
        setTitleBar(R.id.toolbar, R.drawable.titlebar_red_shadow, TitleBarOptions.Builder()
                .leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
                    override fun onClick(view: View) {
                        onBackPressed()
                    }
                })
                .titleAction(DefaultTabHostOptions(this, mTabDataHost, true))
                .rightAction(object : DefaultButtonOptions(this, R.drawable.nim_titlebar_standard_red_btn_selector, R.string.red_packet_help, R.color.white) {
                    override fun onClick(view: View?) {
                        RedPacketHelpActivity.start(this@C2CRedPacketListActivity)
                    }
                })
                .build())
        constructTabHost()
    }

    private fun constructTabHost() {
        val tabLayout = titleBar.getTitleView<CustomTabHostBar>()
        val receivedTab = tabLayout.findViewById<View>(R.id.received_list)
        receivedTab.setOnClickListener(this)
        val sentTab = tabLayout.findViewById<View>(R.id.sent_list)
        sentTab.setOnClickListener(this)

        tabHost.put(TAB_RECEIVED, receivedTab)
        tabHost.put(TAB_SENT, sentTab)

        setTabSelected(mCurrentTabIndex)
        this.tabLayout = tabLayout
    }

    private fun setTabSelected(index : Int) {
        mCurrentTabIndex = index
        for (i in 0 until tabHost.size()) {
            tabHost[i]?.isSelected = i == index
        }
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.received_list -> {
                if (mCurrentTabIndex == TAB_RECEIVED) {
                    return
                }
                setTabSelected(TAB_RECEIVED)
                switchContent(receivedFragment)
                if (tabLayout is CustomTabHostBar) {
                    (tabLayout as CustomTabHostBar).playAnimation(1, 0)
                }
            }
            R.id.sent_list -> {
                if (mCurrentTabIndex == TAB_SENT) {
                    return
                }
                setTabSelected(TAB_SENT)
                switchContent(sentFragment)
                if (tabLayout is CustomTabHostBar) {
                    (tabLayout as CustomTabHostBar).playAnimation(0, 1)
                }
            }
            else -> {
            }
        }
    }
}