package com.netease.nim.uikit.business.redpacket.activity

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.GridLayoutManager
import android.view.View
import android.widget.Toast
import com.bullet.messenger.widgetkit.adapter.BaseRecyclerAdapter
import com.bullet.messenger.widgetkit.viewholder.BaseRecyclerViewHolder
import com.bullet.messenger.widgetkit.viewholder.ViewHolderFactory
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.presenter.GetCashPresenter
import com.netease.nim.uikit.business.redpacket.presenter.WalletPresenter
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import com.smartisan.libstyle.toast.NimToast
import com.tencent.mm.opensdk.utils.Log
import kotlinx.android.synthetic.main.activity_b2c_get_cash.*
import kotlinx.android.synthetic.main.b2c_amount_list_item_layout.view.*
import kotlinx.android.synthetic.main.b2c_info_header_layout.view.*

/**
 * Created by lan on 2018/9/5.
 */
class GetCashActivity : UI(), GetCashPresenter.GetCashView, WalletPresenter.WalletView {

    companion object {
        private var TAG = GetCashActivity::class.java.simpleName
        fun start(context: Context) {
            context.startActivity(Intent(context, GetCashActivity::class.java))
        }
    }

    private val getCashPresenter by lazy {
        GetCashPresenter(this)
    }

    private val walletPresenter by lazy {
        WalletPresenter(this)
    }

    private val amountListAdapter = AmountListAdapter()

    private var checkAmount: AmountModel? = null

    private var balance: Double = 0.00

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_b2c_get_cash)
        initTitleBar()
        walletPresenter.onAttach(this)
        walletPresenter.initWalletData()
        getCashPresenter.onAttach(this)

        amountListAdapter.viewHolderConfig.itemClickListener = { model, _ ->
            if (model is AmountModel) {
                checkAmount?.check = false
                checkAmount = model
                checkAmount?.check = true

                amountListAdapter.notifyDataSetChanged()

                ruleView.text = checkAmount?.rule
                changeSendBtnStatus()

            } else {
                Log.i(TAG, "model is not bind event")
            }
        }

        amountList.apply {
            layoutManager = GridLayoutManager(this@GetCashActivity, 4)
            adapter = amountListAdapter
        }

        getCashButton.setOnClickListener {
            getCashPresenter.withdraw(checkAmount?.amount)
        }
    }

    override fun loadedCashList(amountList: List<AmountModel>) {
        amountListAdapter.addItems(amountList)
    }

    override fun getCashSuccess(amount: String?) {
        balance = amount?.toDoubleOrNull() ?: 0.00
        sumLayout.sumAmountTextView.text = amount
        NimToast.makeToast(this, R.string.get_cash_success, Toast.LENGTH_SHORT).show()
    }

    override fun fillData(sumAmount: String?) {
        balance = sumAmount?.toDoubleOrNull() ?: 0.00
        changeSendBtnStatus()
        sumLayout.apply {
            sumAmountTextView.text = sumAmount ?: getString(R.string.default_money)
            topTitleView.setText(R.string.current_amount)
        }

        getCashPresenter.initCashList()
    }

    private fun changeSendBtnStatus(){
        getCashButton.isEnabled = (checkAmount?.enable == true && (checkAmount?.getDoubleAmount() ?:0.00) <= balance)
    }

    override fun onDestroy() {
        super.onDestroy()
        getCashPresenter.onDetach()
        walletPresenter.onDetach()
    }

    fun initTitleBar() {
        toolbar.setBackgroundResource(R.drawable.titlebar_red_bg)
        val titleOptions = TitleBarOptions.Builder()
        titleOptions
                .leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
                    override fun onClick(view: View) {
                        onBackPressed()
                    }
                })
                .titleAction(DefaultTitleOptions(this, R.string.withdraw_cash, R.color.white))
                .build()

        toolbar.build(titleOptions.build())
    }

    data class AmountModel(var amount: String?,
                           var rule: String?,
                           var enable: Boolean?,
                           var check: Boolean?){
        fun getDoubleAmount(): Double {
            return amount?.toDoubleOrNull() ?: 0.00
        }
    }

    class AmountViewHolder(itemView: View) : BaseRecyclerViewHolder<AmountModel>(itemView) {

        override fun configView(model: AmountModel) {
            itemView.apply {
                model.apply {
                    contentView.text = itemView.context.getString(R.string.yuan, amount)
                    checkView.visibility = if (check == true) View.VISIBLE else View.INVISIBLE
                }
            }
        }

        companion object : ViewHolderFactory<AmountViewHolder> {
            override fun getLayoutId(): Int = R.layout.b2c_amount_list_item_layout

            override fun getViewHolderClass(): Class<*> = AmountViewHolder::class.java
        }
    }

    class AmountListAdapter : BaseRecyclerAdapter() {
        override fun getViewHolderFactoryByModel(model: Any): ViewHolderFactory<BaseRecyclerViewHolder<*>>? {
            return AmountViewHolder.Companion
        }
    }

}