package com.netease.nim.uikit.business.redpacket.activity

import android.content.Context
import android.content.Intent
import android.graphics.Typeface
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.SpannableString
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.text.style.StyleSpan
import android.view.View
import android.widget.ListAdapter
import android.widget.Toast
import cn.testin.analysis.TestinApi
import com.bullet.messenger.business.base.ABTestinConstant.*
import com.bullet.point.grpc.CreateInviteUrlBatchRequest
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.contact.SurnameListenerWrapper
import com.netease.nim.uikit.business.contact.core.item.ContactItem
import com.netease.nim.uikit.business.contact.core.item.ItemTypes
import com.netease.nim.uikit.business.contact.core.model.ContactDataAdapter
import com.netease.nim.uikit.business.contact.core.model.ContactSearchStrategy
import com.netease.nim.uikit.business.contact.core.provider.ContactDataProvider
import com.netease.nim.uikit.business.contact.core.viewholder.LabelHolder
import com.netease.nim.uikit.business.preference.PointConfig
import com.netease.nim.uikit.business.redpacket.viewholder.CheckContactHolder
import com.netease.nim.uikit.business.redpacket.viewholder.SelectContactHolder
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import com.netease.nim.uikit.common.rx.DisposableCollector
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import com.netease.nim.uikit.common.ui.quickbar.QuickBarEx2
import com.smartisan.libstyle.toast.NimToast
import com.tencent.mm.opensdk.utils.Log
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_phone_contacts.*
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler

class PhoneContactsActivity : UI() {

    companion object {
        private var TAG = PhoneContactsActivity::class.java.simpleName
        fun starter(context: Context) {
            val intent = Intent(context, PhoneContactsActivity::class.java)
            context.startActivity(intent)
        }
    }

    private val dataProvider by lazy {
        ContactDataProvider(ItemTypes.PHONE_CONTACT)
    }

    private var adapter: ContactDataAdapter? = null

    private var isMultiSelect = false
    private var isSelectAll = false

    private val disposableCollector by lazy {
        DisposableCollector()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_phone_contacts)
        initTitleBar()

        quickContactBar.initLetters()

        quickContactBar.setQBListener(object : QuickBarEx2.QBListener {
            override fun onLetterGridHidden() {
            }

            override fun onLetterGridShow() {
            }

            override fun onLetterChanged(letter: String?, type: Int): Boolean {
                letter ?: return false
                val datas = adapter?.datas ?: return false
                val mapABC = datas.indexes
                if (mapABC != null) {

                    var index = -1
                    if (mapABC.containsKey(letter)) {
                        index = mapABC[letter] ?: 0
                    }
                    if (index < 0) {
                        return false
                    }

                    index += contactListView.headerViewsCount
                    if (index >= 0 && index < contactListView.count) {
                        contactListView.setSelectionFromTop(index, 0)
                        return true
                    }
                }

                return false
            }

        })

        quickContactBar.setSurnameListener(object : SurnameListenerWrapper() {
            override fun getTargetListAdapter(): ListAdapter {
                return contactListView.adapter
            }

            override fun onSurnameClicked(position: Int) {
                contactListView.setSelection(position)
            }

        })

        quickContactBar.setOnTouchListener { _, _ -> false }

        DialogMaker.showProgressDialog(this, null)
        TestinApi.asyncGetExperiments { success ->
            DialogMaker.dismissProgressDialog()
            if (success) {
                isMultiSelect = TestinApi.getIntegerFlag(SMS_INVITE_TYPE, SMS_INVITE_TYPE_DEFAULT) == SMS_INVITE_TYPE_MULTI
            }
            initView()
        }

    }

    fun initView() {
        rootLayout.setBackgroundResource(R.color.transparent)
        adapter = ContactDataAdapter(this, ContactSearchStrategy(), dataProvider).apply {
            addViewHolder(ItemTypes.LABEL, LabelHolder::class.java)
            if (isMultiSelect) {
                addViewHolder(ItemTypes.PHONE_CONTACT, SelectContactHolder::class.java)
            } else {
                addViewHolder(ItemTypes.PHONE_CONTACT, CheckContactHolder::class.java)
            }

            isRapidReplyEnable = false
            isQuickBarEnable = true
        }
        contactListView.adapter = adapter
        adapter?.load(false)

        adapter?.setItemClickCallback {
            if (isMultiSelect) {
                updateTitleByMultiSelect()
            } else {
                updateTitle()
            }
        }

        adapter?.setUpdateCallback {
            if (isMultiSelect) {
                updateTitleByMultiSelect()
            } else {
                updateTitle()
            }
        }

        if (isMultiSelect) {
            topBarLayout.visibility = View.GONE
            bottomLineView.visibility = View.VISIBLE
            selectAllLayout.visibility = View.VISIBLE
            bottomBarLayout.visibility = View.VISIBLE

            selectAllLayout.setOnClickListener {
                selectAllView.isSelected = !selectAllView.isSelected
                selectAll(selectAllView.isSelected)
                updateTitleByMultiSelect()
            }

            sendInviteBtn.setOnClickListener {
                sendAll()
            }

        } else {
            topBarLayout.visibility = View.VISIBLE
            bottomLineView.visibility = View.GONE
            selectAllLayout.visibility = View.GONE
            bottomBarLayout.visibility = View.GONE
        }
    }

    fun initTitleBar() {
        toolbar.setBackgroundResource(R.drawable.titlebar_red_bg)
        val titleOptions = TitleBarOptions.Builder()

        titleOptions
                .leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
                    override fun onClick(view: View) {
                        onBackPressed()
                    }
                })
                .titleAction(DefaultTitleOptions(this, R.string.sms_invite, R.color.white))
                .build()

        setTitleBar(R.id.toolbar, titleOptions.build())
    }

    private fun sendAll() {
        val requestBuilder = CreateInviteUrlBatchRequest.newBuilder()
        adapter?.datas?.items?.forEachIndexed { index, item ->
            if (item is ContactItem && item.contact.isSendInviteSMS) {
                val contact = CreateInviteUrlBatchRequest.CreateInviteUrl.newBuilder()
                        .setInviteeName(item.contact.displayName)
                        .setInviteeTelephone(item.contact.phoneNumber)
                requestBuilder.setUsers(index, contact)
            }
        }

        Observable.fromCallable {
            ProtoBuffManager.getInstance().doB2CRedPacketStub().createInviteUrlBatch(requestBuilder.build())
        }.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer {
                    Log.i(TAG, "invite count:" + it.inviteUrlsCount)
                    NimToast.makeToast(this, R.string.invite_success, Toast.LENGTH_SHORT).show()
                    finish()
                }, RxGrpcErrorHandler(this))
                ?.let { disposableCollector.collect(it) }
    }

    private fun updateTitle() {
        var checkedCount = 0
        var sumCount = 0
        adapter?.datas?.items?.forEach {
            if (it is ContactItem) {
                sumCount++
                if (it.contact.isSendInviteSMS) {
                    checkedCount++
                }
            }
        }

        fillTitle(sumCount)
    }

    private fun updateTitleByMultiSelect() {
        var checkedCount = 0
        var sumCount = 0
        adapter?.datas?.items?.forEach {
            if (it is ContactItem) {
                sumCount++
                if (it.isChecked) {
                    checkedCount++
                }
            }
        }

        selectAllView.isSelected = sumCount == checkedCount
        sendInviteBtn.isEnabled = checkedCount > 0
        fillTitleByMultiSelect(sumCount, checkedCount)
    }

    private fun selectAll(isSelectAll: Boolean) {
        var sumCount = 0
        adapter?.datas?.items?.forEach {
            if (it is ContactItem) {
                if (isSelectAll) {
                    sumCount++
                }
                it.isChecked = isSelectAll
            }
        }
        fillTitle(sumCount)
        adapter?.notifyDataSetChanged()
    }

    private fun fillTitle(sumCount: Int) {
        val sumReward = sumCount * PointConfig.getInstance().inviteBonusEach
        val title = getString(R.string.sum_reward, sumReward)

        val lastStart = title.lastIndexOf(sumReward.toString())
        val lastEnd = lastStart + sumReward.toString().length

        val spannableString = SpannableString(title)

        spannableString.setSpan(StyleSpan(Typeface.BOLD), lastStart, lastEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(ForegroundColorSpan(ContextCompat.getColor(this, R.color.red_packet_text_color)), lastStart, lastEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)

        inviteLeftNumber.text = spannableString
    }

    private fun fillTitleByMultiSelect(sumCount: Int, checkedCount: Int) {
        val sumReward = sumCount * 2

        val title1 = getString(R.string.have_invited_today, sumReward, checkedCount, sumCount)

        val firstStart = title1.indexOf(sumReward.toString())
        val firstEnd = firstStart + sumReward.toString().length

        val spannableString = SpannableString(title1)
        spannableString.setSpan(StyleSpan(Typeface.BOLD), firstStart, firstEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(ForegroundColorSpan(ContextCompat.getColor(this, R.color.red_packet_text_color)), firstStart, firstEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)

        selectAllTextView.text = spannableString

        val checkedReward = checkedCount * 2
        val title2 = getString(R.string.select_all_sum_reward, checkedReward)

        sendInviteBtn.text = title2

        freeSMSTextView.text = replaceBySpannable(getString(R.string.sms_purchase_free_guide), getString(R.string.free), ContextCompat.getColor(this, R.color.red_packet_text_color))
    }

    private fun replaceBySpannable(originStr: String, replaceStr: String, color: Int): SpannableString {
        val firstStart = originStr.lastIndexOf(replaceStr)
        val firstEnd = firstStart + replaceStr.length

        return SpannableString(originStr).apply {
            setSpan(StyleSpan(Typeface.BOLD), firstStart, firstEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
            setSpan(ForegroundColorSpan(color), firstStart, firstEnd, Spanned.SPAN_INCLUSIVE_EXCLUSIVE)
        }

    }

    override fun onDestroy() {
        super.onDestroy()
        disposableCollector.clearDisposable()
    }
}