package com.netease.nim.uikit.business.redpacket.activity

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.InputFilter
import android.text.Spanned
import android.text.TextWatcher
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.Toast
import com.bullet.messenger.business.base.DataAnalytics
import com.bullet.messenger.business.base.DataTrackEvents
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.preference.RedPacketConfig
import com.netease.nim.uikit.business.redpacket.AmountChecker
import com.netease.nim.uikit.business.redpacket.SingleClick
import com.netease.nim.uikit.business.redpacket.presenter.SendRedPacketPresenter
import com.netease.nim.uikit.business.reply.MsgServiceUtil
import com.netease.nim.uikit.business.session.constant.Extras
import com.netease.nim.uikit.business.session.constant.Extras.EXTRA_SESSION_ID
import com.netease.nim.uikit.business.session.constant.Extras.EXTRA_SESSION_TYPE
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.activity.titlebar.DefaultButtonOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.common.util.string.MatchRegexUtils
import com.netease.nim.uikit.common.util.sys.SoftKeyboardUtil
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import kotlinx.android.synthetic.main.activity_send_red_packet.*
import kotlinx.android.synthetic.main.red_packet_edit_layout.view.*
import kotlinx.android.synthetic.main.red_packet_tab_layout.view.*
import org.json.JSONObject
import java.text.DecimalFormat

class SendRedPacketActivity : UI() {

    companion object {
        private const val DEFAULT_AMOUNT = 0.00
        fun starter(context: Context?, sessionId: String?, sessionType: SessionTypeEnum) {
            var soure = ""
            if (context is Activity) {
                soure = context.intent.getStringExtra(Extras.EXTRA_CLICK_SOURCE)
            }
            val starterIntent = Intent(context, SendRedPacketActivity::class.java).apply {
                putExtra(Extras.EXTRA_SESSION_ID, sessionId)
                putExtra(Extras.EXTRA_SESSION_TYPE, sessionType.value)
                putExtra(Extras.EXTRA_CLICK_SOURCE, soure)
            }

            context?.startActivity(starterIntent)
        }
    }


    private val sessionId by lazy {
        intent.getStringExtra(EXTRA_SESSION_ID)
    }

    private val sessionType by lazy {
        SessionTypeEnum.typeOfValue(intent.getIntExtra(EXTRA_SESSION_TYPE, SessionTypeEnum.P2P.value))
    }

    private val sendRedPacketPresenter by lazy {
        SendRedPacketPresenter(sessionId, sessionType)
    }

    private val titleLeftAction by lazy {
        object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
            override fun onClick(view: View) {
                onBackPressed()
                showKeyboard(false)
            }
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        sendEventData(false)
    }

    private val titleRightAction by lazy {
        object : DefaultButtonOptions(this, R.drawable.nim_titlebar_standard_red_btn_selector, R.string.red_packet_list, R.color.white) {
            override fun onClick(view: View?) {
                C2CRedPacketListActivity.start(this@SendRedPacketActivity)
            }
        }
    }

    private var redPacketType = RedPacketTypeV2.GROUP_LUCKY

    private val sendEvent by lazy {
        object: SingleClick(){
            override fun performClick(v: View?) {
                when {
                    MsgServiceUtil.isInBlackList(sessionId) -> NimToast.makeToast(this@SendRedPacketActivity, R.string.p2p_send_message_not_allow_because_not_friend, Toast.LENGTH_LONG).show()
                    sessionType == SessionTypeEnum.P2P -> {
                        sendRedPacketPresenter.sendRedPacket(getAmount(), getMsg())
                    }
                    sessionType == SessionTypeEnum.Team -> {
                        sendRedPacketPresenter.sendRedPacketV2(getAmount(), getCount(), getMsg(), sessionName, redPacketType)
                    }
                    else -> {

                    }
                }
            }
        }
    }

    private val typeEvent by lazy {
        object : View.OnClickListener {
            var checkedTab: View? = luckyTab
            override fun onClick(v: View?) {
                if (checkedTab == v) {
                    return
                }
                checkedTab?.isSelected = false
                checkedTab = v
                checkedTab?.isSelected = true

                when (checkedTab) {
                    avgTab -> {
                        redPacketType = RedPacketTypeV2.GROUP_AVG
                        amountLayout.title.setText(R.string.single_amount)
                        descRedTypeView.setText(R.string.avg_red_packet_desc)
                    }
                    luckyTab -> {
                        redPacketType = RedPacketTypeV2.GROUP_LUCKY
                        amountLayout.title.setText(R.string.sum_amount)
                        descRedTypeView.setText(R.string.lucky_red_packet_desc)
                    }
                }

                amountLimitTip.visibility = View.GONE
                fillAmountStr(amountLayout.editText.text.toString(), countLayout.editText.text.toString())
            }
        }
    }

    private var sessionName: String? = null

    private val decimalFormat by lazy {
        DecimalFormat("0.00")
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_send_red_packet)
        initTitleBar()

        // 空格、回车和字数规则
        MatchRegexUtils.initEditTextMaxLengthListener(messageEditText, 48, true)

        // 金额输入规则
        amountLayout.editText.apply {
            addTextChangedListener(CompatHint(amountLayout.hint))
            filters = arrayOf(RPMoneyEditConfig(), InputFilter.LengthFilter(9))
            isFocusableInTouchMode = true
            requestFocus()
        }

        sendBtn.setOnClickListener(sendEvent)
        sendBtn.isEnabled = false

        displayAmountLayout.setOnClickListener {
            amountLayout.editText.isFocusableInTouchMode = true
            amountLayout.editText.requestFocus()
            SoftKeyboardUtil.showSoftKeyboard(amountLayout.editText)
        }

        messageEditText.apply {
            filters = arrayOf(RPMsgEditConfig())
            hint = getDefaultMsg()
        }

        scrollView.setOnScrollListener { _, t, _, oldt ->
            if (oldt - t > 10) {
                SoftKeyboardUtil.hideSoftKeyboard(scrollView)
            }
        }

        // 开启群红包模式
        if (isTeamType()) {
            teamRedPacketSetting()
        }

        sendRedPacketPresenter.onAttach(this)
    }

    override fun onDestroy() {
        super.onDestroy()
        sendRedPacketPresenter.onDetach()
    }

    fun initTitleBar() {
        findViewById<View>(R.id.toolbar).setBackgroundResource(R.drawable.titlebar_red_bg)
        val builder = TitleBarOptions.Builder()
                .leftAction(titleLeftAction)
                .rightAction(titleRightAction)
                .titleAction(DefaultTitleOptions(this, R.string.send_packet, R.color.white))
        setTitleBar(R.id.toolbar, R.drawable.titlebar_red_shadow, builder.build())
    }

    private fun fillTeamInfo() {
        if (sessionType != SessionTypeEnum.Team) {
            return
        }
        val t = NimUIKit.getTeamProvider().getTeamById(sessionId)
        if (t != null) {
            teamSizeTextView.text = t.memberCount.toString()
            sessionName = t.name
        } else {
            NimUIKit.getTeamProvider().fetchTeamById(sessionId) { success, result, _ ->
                sessionName = result.name
                teamSizeTextView.text = if (success && result != null) {
                    result.memberCount.toString()
                } else {
                    ""
                }
            }
        }
    }


    /**
     * 显示格式化后的红包金额
     * 并且做最大金额校验
     * 如果是平均分模式，金额 = 单个红包 * 红包数量
     */
    private fun fillAmountStr(amountStr: String?, countStr: String?) {

        val amount = amountStr?.toDoubleOrNull() ?: DEFAULT_AMOUNT
        var count = countStr?.toIntOrNull() ?: 0

        var amountSum = DEFAULT_AMOUNT
        var amountSingle = DEFAULT_AMOUNT

        if (!isTeamType()) {
            amountSum = amount
            amountSingle = amount
            count = 1
        } else if (typeEvent.checkedTab == avgTab) {
            amountSum = amount * count
            amountSingle = amount

        } else if (typeEvent.checkedTab == luckyTab) {
            amountSum = amount
            amountSingle = if (count == 0) {
                DEFAULT_AMOUNT
            } else {
                amount / count
            }
        }

        displayAccount.text = decimalFormat.format(amountSum)

        checkLimit(amountSingle, count, amountSum)
    }

    private fun checkLimit(amount: Double, count: Int, amountSum: Double) {
        when {
        // 红包总额限制
            amountSum > getAmountSumLimit() -> {
                amountLimitTip.visibility = View.VISIBLE
                amountLimitTip.text = getAmountSumLimitText()
                sendBtn.isEnabled = false
            }
        // 单个红包金额限制
            amount != 0.0 && amount < getPerAmountLowerLimit() -> {
                amountLimitTip.visibility = View.VISIBLE
                amountLimitTip.text = getString(R.string.red_packet_amount_lower_limit, getPerAmountLowerLimit().toString())
                sendBtn.isEnabled = false
            }
            isTeamType() && amount > RedPacketConfig.getInstance().getC2GOneAmountUpperLimit() -> {
                amountLimitTip.visibility = View.VISIBLE
                amountLimitTip.text = getString(R.string.red_packet_amount_limit,
                        RedPacketConfig.getInstance().getC2GOneAmountUpperLimit())
                sendBtn.isEnabled = false
            }
        // 红包数量限制
            count > getCountLimit() -> {
                amountLimitTip.visibility = View.VISIBLE
                amountLimitTip.text = getString(R.string.red_packet_count_limit, getCountLimit())
                sendBtn.isEnabled = false
            }

            count == 0 -> {
                amountLimitTip.visibility = View.GONE
                sendBtn.isEnabled = false
            }
            else -> {
                amountLimitTip.visibility = View.GONE
                sendBtn.isEnabled = amount != 0.0
            }
        }

    }

    /**
     * 获取配置中的红包总额上限
     */
    private fun getAmountSumLimit(): Int {
        return if (isTeamType()) {
            RedPacketConfig.getInstance().getC2GAmountUpperLimit()
        } else {
            RedPacketConfig.getInstance().getC2CAmountUpperLimit()
        }
    }

    private fun getAmountSumLimitText(): String {
        return if (isTeamType()) {
            getString(R.string.red_packet_amount_limit_02, getAmountSumLimit())
        } else {
            getString(R.string.red_packet_amount_limit, getAmountSumLimit())
        }
    }

    private fun getPerAmountLowerLimit(): Double {
        return if (isTeamType()) {
            RedPacketConfig.getInstance().getC2GOneAmountLowerLimit()
        } else {
            RedPacketConfig.getInstance().getC2CAmountLowerLimit()
        }
    }

    private fun getCountLimit(): Int {
        return RedPacketConfig.getInstance().getC2GNumberLimit()
    }

    private fun getDefaultMsg(): String {
        return if (isTeamType()) {
            RedPacketConfig.getInstance().getC2GDefaultDesc()
        } else {
            RedPacketConfig.getInstance().getC2CDefaultDesc()
        }
    }

    private fun isTeamType(): Boolean {
        return sessionType == SessionTypeEnum.Team
    }

    private fun teamRedPacketSetting() {
        fillTeamInfo()
        teamSizeLayout.visibility = View.VISIBLE
        redPacketTypeLayout.visibility = View.VISIBLE

        countLayout.apply {
            visibility = View.VISIBLE
            editText.addTextChangedListener(CompatHint(hint))
            editText.filters = arrayOf(RPCountEditConfig())
            editText.inputType = EditorInfo.TYPE_CLASS_NUMBER
            hint.text = "0"
            unit.setText(R.string.unit_for_red_packet)
            title.setText(R.string.red_packet_count)
        }

        amountLayout.title.setText(R.string.sum_amount)

        avgTab.setOnClickListener(typeEvent)
        luckyTab.setOnClickListener(typeEvent)

        avgTab.isSelected = false
        luckyTab.isSelected = true

        avgTab.contentView.setText(R.string.average_red_packet)
        luckyTab.contentView.setText(R.string.random_red_packet)

        descRedTypeView.visibility = View.VISIBLE
        descRedTypeView.setText(R.string.lucky_red_packet_desc)
    }

    private fun getMsg(): String? {
        return messageEditText.text?.toString() ?: getDefaultMsg()
    }

    private fun getAmount(): String?{
        return amountLayout.editText.text?.toString()
    }

    private fun getCount(): Int{
        return countLayout.editText.text?.toString()?.toIntOrNull() ?: 0
    }


    inner class CompatHint(private val hintView: View) : TextWatcher {

        override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
            val str: CharSequence = s

            // EditText hint 的兼容处理
            hintView.visibility = if (str.isNotEmpty()) {
                View.INVISIBLE
            } else {
                View.VISIBLE
            }
        }

        override fun afterTextChanged(s: Editable?) {
            fillAmountStr(amountLayout.editText.text.toString(), countLayout.editText.text.toString())
        }

        override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
        }
    }

    /**
     * 红包金额的输入规则
     */
    inner class RPMoneyEditConfig : InputFilter {
        override fun filter(str: CharSequence?, start: Int, end: Int, dest: Spanned?, dstart: Int, dend: Int): CharSequence {
            dest ?: return ""
            str ?: return ""

            if (amountLayout.editText.selectionEnd != dest.length) {
                return ""
            }

            //首位为小数点或者0
            if (dest.isEmpty() && (str == "." || str == "0")) {
                return "0."
            }

            if (dest.length == 1 && dest.contains("0") && str != ".") {
                return ""
            }

            //小数点后只能输入两位
            if (dest.contains(".")) {
                return if (dest.length - dest.indexOf(".") > 2) {
                    ""
                } else {
                    str
                }
            }


            //小数点前只能输入六位
            if (dest.length == 6) {
                if (str != ".") {
                    return ""
                }
            }
            return str
        }
    }

    /**
     * 红包数量的输入规则
     */
    inner class RPCountEditConfig : InputFilter {
        override fun filter(str: CharSequence?, start: Int, end: Int, dest: Spanned?, dstart: Int, dend: Int): CharSequence {
            str ?: return ""
            dest ?: return ""

            if (countLayout.editText.selectionEnd != dest.length) {
                return ""
            }

            if (dest.isEmpty() && str == "0") {
                return ""
            }

            if (dest.length == 4) {
                return ""
            }

            return str
        }
    }

    /**
     * 红包描述的输入规则
     */
    inner class RPMsgEditConfig : InputFilter {
        override fun filter(source: CharSequence?, start: Int, end: Int, dest: Spanned?, dstart: Int, dend: Int): CharSequence {
            source ?: return ""
            if (source.contains("\n")) {
                return source.replace(Regex("\n"), "")
            }
            return source
        }
    }

    internal fun sendEventData(send: Boolean) {
        var obj = JSONObject()
        obj.put(DataTrackEvents.SOURCE_FOR_CLICKING, intent.getStringExtra(Extras.EXTRA_CLICK_SOURCE))
        obj.put(DataTrackEvents.SENT_SUCESSFULLY, send)
        DataAnalytics.getInstance().track(DataTrackEvents.CLICK_REDPACKET_ICON, obj)
    }
}