package com.netease.nim.uikit.business.redpacket.debug

import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.format.DateFormat
import android.widget.Toast
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.database.RPDebugDBHelper
import com.netease.nim.uikit.common.activity.UI
import com.netease.nim.uikit.common.rx.DisposableCollector
import com.smartisan.libstyle.toast.NimToast
import com.tencent.mm.opensdk.utils.Log
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_order_list.*


class OrderListActivity : UI() {

    companion object {
        fun starter(context: Context) {
            val starterIntent = Intent(context, OrderListActivity::class.java)
            context.startActivity(starterIntent)
        }
    }

    private val tag = OrderListActivity::class.java.simpleName

    private val orderAdapter = OrderAdapter()

    private val collector = DisposableCollector()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_order_list)

        orderListView.layoutManager = LinearLayoutManager(this)
        orderListView.adapter = orderAdapter

        orderAdapter.viewHolderConfig.itemClickListener = { model, _ ->
            copyMessage(model.toString())
            NimToast.makeToast(this, "订单信息已复制到剪切板", Toast.LENGTH_LONG).show()
        }


        collector.collect {
            Observable.fromCallable {
                RPDebugDBHelper.getAllOrders()
                        .map {
                            it.apply {
                                formatTime = DateFormat.format("yyyy-MM-dd, HH:mm:ss", time).toString()
                            }
                        }
            }.subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({
                        orderAdapter.addItems(it, true)
                    }, {
                        Log.e(tag, it.message)
                    })
        }


    }

    override fun onDestroy() {
        super.onDestroy()
        collector.clearDisposable()
    }

    private fun copyMessage(message: String) {
        // 应用菜单弹窗复制时，不开启快速分享
        val cm = getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
// 将文本内容放到系统剪贴板里。
        val clipData = ClipData.newPlainText(tag, message)
        cm.primaryClip = clipData
    }
}