package com.netease.nim.uikit.business.redpacket.fragment

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.graphics.Rect
import android.os.Bundle
import android.view.View
import android.view.animation.AccelerateDecelerateInterpolator
import com.netease.nim.uikit.business.redpacket.*
import com.netease.nim.uikit.common.fragment.TFragment

abstract class BaseRedPacketInfoFragment : TFragment() {

    var rpInfoFragLifeCallback: RpInfoFragLifeCallback? = null

    abstract fun isForAnimation():Boolean
    abstract fun getRootLayout():View?
    abstract fun getAmountText():View?
    abstract fun getAvatarView():View?

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        if(isForAnimation()){
            getRootLayout()?.alpha = 0f
            postRunnable {
                rpInfoFragLifeCallback?.onViewCreated(this)
            }
        }
    }

    fun startAnimation() {
        val r1 = ObjectAnimator.ofFloat(getRootLayout(), View.ALPHA, ListAlphaStart, ListAlphaEnd).apply {
            duration = AnimDuration3
            addListener(object : AnimatorListenerAdapter() {
                override fun onAnimationStart(animation: Animator?) {
                    val rect = Rect()
                    getAvatarView()?.getGlobalVisibleRect(rect)
                    rpInfoFragLifeCallback?.onAnimationStart(this@BaseRedPacketInfoFragment, rect)
                }

                override fun onAnimationEnd(animation: Animator?) {
                    rpInfoFragLifeCallback?.onAnimationEnd(this@BaseRedPacketInfoFragment)
                }
            })
        }


        val moneyTextView = getAmountText()
        val t1 = ObjectAnimator.ofFloat(moneyTextView, View.SCALE_X, AmountScaleStart, AmountScaleEnd)
        val t2 = ObjectAnimator.ofFloat(moneyTextView, View.SCALE_Y, AmountScaleStart, AmountScaleEnd)
        val t3 = ObjectAnimator.ofFloat(moneyTextView, View.ALPHA, AmountAlphaStart, AmountAlphaEnd)
        val t4 = ObjectAnimator.ofFloat(moneyTextView, View.SCALE_X, AmountScaleEnd, AmountScaleEnd2)
        val t5 = ObjectAnimator.ofFloat(moneyTextView, View.SCALE_Y, AmountScaleEnd, AmountScaleEnd2)

        t3.duration = AnimDuration2
        t1.duration = AnimDuration2
        t2.duration = AnimDuration2
        t4.duration = AnimDuration1
        t5.duration = AnimDuration1

        t1.interpolator = AccelerateDecelerateInterpolator()
        t2.interpolator = AccelerateDecelerateInterpolator()
        t4.interpolator = AccelerateDecelerateInterpolator()
        t5.interpolator = AccelerateDecelerateInterpolator()

        val animatorSet = AnimatorSet()
        animatorSet.play(t3).with(t1).with(t2).before(t5)
        animatorSet.play(t5).with(t4)

        r1.start()
        animatorSet.start()
    }

    open class RpInfoFragLifeCallback {
        open fun onViewCreated(frag: BaseRedPacketInfoFragment){}
        open fun onAnimationStart(frag: BaseRedPacketInfoFragment, avatarRect: Rect) {}
        open fun onAnimationEnd(frag: BaseRedPacketInfoFragment) {}
    }
}