package com.netease.nim.uikit.business.redpacket.fragment

import android.content.Context
import android.graphics.Typeface
import android.os.Bundle
import android.support.v7.widget.RecyclerView
import android.text.Spannable
import android.text.SpannableString
import android.text.format.DateFormat
import android.text.style.StyleSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.bullet.redpacket.c2c.grpc.GetRedPacketInfoRequest
import com.bullet.redpacket.c2c.v2.grpc.GetRecordListV2Request
import com.bullet.redpacket.c2c.v2.grpc.GetRecordListV2Response
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.bullet.messenger.widgetkit.listener.SingleItemClick
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.favorite.view.PullUpToRefreshListView
import com.netease.nim.uikit.business.redpacket.AmountUtil
import com.netease.nim.uikit.business.redpacket.activity.C2CRedPacketListActivity
import com.netease.nim.uikit.business.redpacket.activity.RedPacketInfoActivity
import com.netease.nim.uikit.business.redpacket.activity.RedPacketInfoC2CActivity
import com.netease.nim.uikit.business.redpacket.request.RPC2CReqSender
import com.netease.nim.uikit.business.redpacket.request.RedPacketReqSender
import com.netease.nim.uikit.business.team.helper.TeamHelper
import com.netease.nim.uikit.common.fragment.TFragment
import com.netease.nim.uikit.common.rx.LoadingDisposableCollector
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import com.netease.nim.uikit.common.ui.imageview.HeadImageView
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.common.ui.widget.imageloader.AvatarImageLoader
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.smartisan.pullToRefresh.PullToRefreshBaseView
import io.reactivex.functions.Consumer
import com.bullet.libcommonutil.log.Logger
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.util.*

/**
 * C2C红包收支明细
 * Created by lan on 2018/9/17.
 */
class C2CRedPacketListFragment : TFragment(), PullToRefreshBaseView.RefreshListener {

    private val TAG = C2CRedPacketListFragment::class.java.simpleName
    private var mAdapter: PacketListAdapter? = null
    private val mData = ArrayList<GetRecordListV2Response.RedPacketRecordInfoOrBuilder>()
    private var type = GetRecordListV2Request.RecordType.IN
    private var page = 1
    private var listView: PullUpToRefreshListView? = null
    private var headerView: View? = null
    private var firstLoad = true
    private var hasMore = true
    private val rpSender by lazy { RedPacketReqSender() }
    private val rpC2CReqSender by lazy { RPC2CReqSender() }
    private val disposableCollection by lazy {
        LoadingDisposableCollector(context)
    }

    companion object {
        fun newInstance(pageMode: Int): C2CRedPacketListFragment {
            val fragment = C2CRedPacketListFragment()
            val bundle = Bundle()
            bundle.putInt("pageMode", pageMode)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.fragment_red_packet_list, container, false)
        type = if (arguments?.getInt("pageMode") == C2CRedPacketListActivity.TAB_RECEIVED)
            GetRecordListV2Request.RecordType.IN else GetRecordListV2Request.RecordType.OUT

        view?.let {
            initView(view)
        }
        return view
    }

    override fun onResume() {
        super.onResume()
        if (firstLoad) {
            loadData(true)
            firstLoad = false
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        disposableCollection.clearDisposable()
    }

    private fun initView(view: View) {
        listView = view.findViewById(R.id.listView)
        mAdapter = PacketListAdapter(activity!!, type, mData)
        listView?.setAdapter(mAdapter)
        listView?.setEnabledPullDownToRefresh(false)
        listView?.setRefreshListener(this)

        generateHeaderView()
        listView?.addHeaderView(headerView)


        listView?.setOnItemClickListener(object : SingleItemClick() {
            override fun performItemClick(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
                val realPosition = position - (listView?.headerViewsCount ?: 0)
                if (realPosition >= 0) {
                    goToInfo(mData[realPosition].fromAccid, mData[realPosition].redPacketId, mData[realPosition].redPacketType)
                }
            }
        })
    }

    private fun goToInfo(fromAccid: String?, redPacketId: String?, redPacketTypeV2: RedPacketTypeV2) {
        fromAccid ?: return
        redPacketId ?: return
        disposableCollection.loading(true)

        if (redPacketTypeV2 == RedPacketTypeV2.C2C) {

            val request = GetRedPacketInfoRequest.newBuilder()
                    .setRedPacketId(redPacketId)
                    .setFromAccid(fromAccid)
                    .build()
            rpC2CReqSender.getRedPacketInfo(request)
                    .subscribe(Consumer { redPacketInfo ->
                        disposableCollection.loading(false)
                        RedPacketInfoC2CActivity.starter(activity!!, redPacketInfo, showDetailBtn = false)
                    }, object : RxGrpcErrorHandler() {
                        override fun error(code: Int, errorMsg: String) {
                            NimToast.makeToast(activity!!, errorMsg, Toast.LENGTH_SHORT).show()
                            disposableCollection.loading(false)
                        }
                    })
        } else if (redPacketTypeV2 == RedPacketTypeV2.GROUP_AVG
                || redPacketTypeV2 == RedPacketTypeV2.GROUP_LUCKY) {

            rpSender.getInfo(fromAccid, redPacketId, redPacketTypeV2)
                    .subscribe(Consumer {
                        disposableCollection.loading(false)
                        RedPacketInfoActivity.starter(activity!!, it, false)
                    }, object : RxGrpcErrorHandler() {
                        override fun error(code: Int, errorMsg: String?) {
                            disposableCollection.loading(false)
                            NimToast.makeToast(activity!!, errorMsg
                                    ?: getString(R.string.load_data_failed), Toast.LENGTH_LONG).show()
                        }
                    })?.let { disposableCollection.collect(it) }
        }

    }

    private fun generateHeaderView() {
        if (headerView == null) {
            headerView = LayoutInflater.from(activity).inflate(R.layout.red_packet_info_layout, null)

            val avatarImageView = headerView?.findViewById<HeadImageView>(R.id.avatarImageView)
            val userNameView = headerView?.findViewById<TextView>(R.id.userNameView)
            val msgTextView = headerView?.findViewById<TextView>(R.id.msgTextView)
            avatarImageView?.loadBuddyAvatar(NimUIKitImpl.getAccount(),true)
            userNameView?.text = NimUIKit.getUserName()

            msgTextView?.visibility = View.INVISIBLE

        }
    }

    private fun fillHeaderView(amount: String, count: Int) {

        val msgTextView = headerView?.findViewById<TextView>(R.id.msgTextView)
        val moneyTextView = headerView?.findViewById<TextView>(R.id.moneyTextView)
        val redPacketDataView = headerView?.findViewById<TextView>(R.id.redPacketDataView)

        if (type == GetRecordListV2Request.RecordType.IN
                && !AmountUtil.isNoMoney(amount)) {
            redPacketDataView?.visibility = View.VISIBLE
        } else {
            redPacketDataView?.visibility = View.INVISIBLE
        }

        msgTextView?.visibility = View.VISIBLE

        val countString = activity?.getString(if (type == GetRecordListV2Request.RecordType.IN) R.string.receive_packet_total else R.string.sent_packet_total, count.toString())
        val spannable = SpannableString(countString)
        spannable.setSpan(StyleSpan(Typeface.BOLD), 4, count.toString().length + 4, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        msgTextView?.text = spannable
        moneyTextView?.text = amount
    }

    private fun loadData(clear: Boolean) {
        if (clear) {
            mData.clear()
        }
        DialogMaker.showProgressDialog(activity, null, false)
        val request = GetRecordListV2Request.newBuilder()
                .setPageNo(page)
                .setPageSize(20)
                .setType(type)
                .build()
        rpSender.getRedPacketRecordList(request)
                .subscribe(Consumer { response ->
                    Logger.i("getRedPacketRecord", response.toString())
                    DialogMaker.dismissProgressDialog()
                    if (view != null) {
                        fillHeaderView(response.totalAmount, response.totalCount)
                        val list = response.redPacketRecordsOrBuilderList

                        hasMore = list.size != 0
                        page++

                        mData.addAll(list)
                        mAdapter?.notifyDataSetChanged()

                        if (!clear) {
                            listView?.finishRefreshing(PullToRefreshBaseView.PULL_UP)
                        }
                    }
                }, object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String?) {
                        Logger.i("getRedPacketRecord", errorMsg + code)
                        DialogMaker.dismissProgressDialog()
                        if (view != null) {
                            NimToast.makeToast(activity!!, "获取明细失败", Toast.LENGTH_SHORT).show()
                            if (!clear) {
                                listView?.finishRefreshing(PullToRefreshBaseView.PULL_UP)
                            }
                        }
                    }
                })?.let { disposableCollection.collect(it) }
    }

    override fun onPullDownToRefresh() {
    }

    override fun onPullUpTpRefresh() {
        if (hasMore) {
            loadData(false)
        } else {
            NimToast.makeToast(activity!!, "没有更多数据啦~", Toast.LENGTH_SHORT).show()
            listView?.finishRefreshing(PullToRefreshBaseView.PULL_UP)
        }
    }

    override fun onPullUpToLoad() {
    }

    class PacketListAdapter(val context: Context,
                            val type: GetRecordListV2Request.RecordType,
                            val mData: ArrayList<GetRecordListV2Response.RedPacketRecordInfoOrBuilder>) : BaseAdapter() {

        override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
            val result: View = convertView
                    ?: LayoutInflater.from(context).inflate(R.layout.item_red_packet_detail, parent, false)
            result.tag = convertView?.tag ?: PacketItemViewHolder(result)

            val viewHolder = result.tag as PacketItemViewHolder
            onBindViewHolder(viewHolder, position)
            return result
        }

        override fun getItem(position: Int): Any {
            return mData[position]
        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getCount(): Int {
            return mData.size
        }

        fun onBindViewHolder(holder: PacketItemViewHolder, position: Int) {
            val data = mData[position]
            if (position == 0) {
                holder.dividerTopIfNeed.visibility = View.VISIBLE
            } else {
                holder.dividerTopIfNeed.visibility = View.GONE
            }

            holder.receiveTime.text = DateFormat.format("MM-dd HH:mm", data.createTime)

            if (type == GetRecordListV2Request.RecordType.IN) {
                holder.receiveAvatar.loadBuddyAvatar(data.fromAccid)
                holder.receiveName.text = TeamHelper.getTeamMemberDisplayNameWithMe(data.sessionId, data.fromAccid)
                holder.receiveStatus.visibility = View.GONE
                holder.receiveAmount.text = String.format("%s 元", data.receivedAmount)
            } else {

                if (data.redPacketType == RedPacketTypeV2.GROUP_AVG
                        || data.redPacketType == RedPacketTypeV2.GROUP_LUCKY) {
                    AvatarImageLoader.getInstance().loadTeamAvatar(holder.receiveAvatar, data.sessionId)
                }else {
                    holder.receiveAvatar.loadBuddyAvatar(data.sessionId)
                }

                holder.receiveName.text = data.sessionName
                holder.receiveStatus.visibility = View.VISIBLE
                holder.receiveAmount.text = String.format("%s 元", data.amount)
            }

            when (data.redPacketType) {
                RedPacketTypeV2.GROUP_AVG -> {
                    holder.typeIconView.visibility = View.VISIBLE
                    holder.typeIconView.setImageResource(R.drawable.icon_jun)
                }

                RedPacketTypeV2.GROUP_LUCKY -> {
                    holder.typeIconView.visibility = View.VISIBLE
                    holder.typeIconView.setImageResource(R.drawable.icon_pin)
                }
                else -> {
                    holder.typeIconView.visibility = View.INVISIBLE
                }
            }

            if (data.redPacketType == RedPacketTypeV2.GROUP_LUCKY
                    || data.redPacketType == RedPacketTypeV2.GROUP_AVG) {

                holder.receiveStatus.text = when (data.status) {
                    RedPacketStatus.FINISH -> {
                        context.getString(R.string.have_got_done)
                    }
                    else -> {
                        context.getString(R.string.is_got_with_count, data.receivedCount, data.count)
                    }
                }

            } else if (data.redPacketType == RedPacketTypeV2.C2C) {
                holder.receiveStatus.text = when (data.status) {
                    RedPacketStatus.FINISH -> {
                        context.getString(R.string.is_got)
                    }

                    RedPacketStatus.EXPIRED -> {
                        context.getString(R.string.is_expired)
                    }
                    else -> {
                        context.getString(R.string.will_get)
                    }
                }
            }
        }
    }

    class PacketItemViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val receiveAvatar: HeadImageView = view.findViewById(R.id.receiveAvatar)
        val receiveName: TextView = view.findViewById(R.id.receiveName)
        val receiveTime: TextView = view.findViewById(R.id.receiveTime)
        val receiveAmount: TextView = view.findViewById(R.id.receiveAmount)
        val receiveStatus: TextView = view.findViewById(R.id.receiveStatus)
        val dividerTopIfNeed: View = view.findViewById(R.id.dividerTopIfNeed)
        val typeIconView: ImageView = view.findViewById(R.id.typeIconView)
    }
}