package com.netease.nim.uikit.business.redpacket.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.InviteUrl
import com.netease.nim.uikit.business.websearch.BrowserDispatcher
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import kotlinx.android.synthetic.main.fragment_rp_info_b2c.*
import kotlinx.android.synthetic.main.fragment_rp_info_b2c_invite.*
import kotlinx.android.synthetic.main.red_packet_info_layout.view.*

class RedPacketInfoB2CFragment : BaseRedPacketInfoFragment() {

    companion object {
        private const val EXTRA_AMOUNT = "EXTRA_AMOUNT"
        private const val EXTRA_FOR_ANIMATION = "EXTRA_FOR_ANIMATION"
        private const val EXTRA_BY_INVITE = "EXTRA_BY_INVITE"

        fun newInstance(amount: String, forAnimation: Boolean, isInvite: Boolean): RedPacketInfoB2CFragment {
            return RedPacketInfoB2CFragment().apply {
                arguments = Bundle().apply {
                    putString(EXTRA_AMOUNT, amount)
                    putBoolean(EXTRA_FOR_ANIMATION, forAnimation)
                    putBoolean(EXTRA_BY_INVITE, true)
                }
            }
        }
    }

    private val resLayoutId by lazy {
        if (isInvite) {
            R.layout.fragment_rp_info_b2c_invite
        } else {
            R.layout.fragment_rp_info_b2c
        }
    }

    private val isInvite by lazy {
        arguments?.getBoolean(EXTRA_BY_INVITE, false) == true
    }

    private val amountText by lazy {
        arguments?.getString(EXTRA_AMOUNT) ?: getString(R.string.default_money)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(resLayoutId, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        if (isInvite) {
            infoLayout.apply {
                moneyTextView.text = arguments?.getString(EXTRA_AMOUNT) ?: getString(R.string.default_money)
                avatarImageView.setImageResource(R.drawable.logo_with_shadow)
                msgTextView.text = getString(R.string.red_packet_new_user)
                userNameView.text = getString(R.string.bullet_red_packet)
            }
        } else {
            moneyTextView.text = amountText
            joinBtn.setOnClickListener {
                BrowserDispatcher.gotoBrowserByB2C(activity, InviteUrl)
            }
        }

        initTitleBar()
    }

    fun initTitleBar() {

        val bar = if (isInvite) inviteToolbar else toolbar
        bar.setBackgroundResource(R.drawable.titlebar_red_bg)
        val titleOptions = TitleBarOptions.Builder()

        titleOptions
                .leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
                    override fun onClick(view: View) {
                        activity?.onBackPressed()
                    }
                })
                .titleAction(DefaultTitleOptions(activity, R.string.invite_red_packet_text, R.color.white))
                .build()

        bar.build(titleOptions.build())
    }

    override fun isForAnimation(): Boolean = arguments?.getBoolean(EXTRA_FOR_ANIMATION) ?: false

    override fun getRootLayout(): View? = if (isInvite) inviteRootLayout else rootLayout

    override fun getAmountText(): View? = if (isInvite) infoLayout.moneyTextView else moneyTextView

    override fun getAvatarView(): View? = if (isInvite) infoLayout.avatarImageView else null
}