package com.netease.nim.uikit.business.redpacket.fragment

import android.content.Context
import android.os.Bundle
import android.support.v4.app.FragmentActivity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.redpacket.activity.C2CRedPacketListActivity
import com.netease.nim.uikit.business.redpacket.presenter.RPInfoPresenter
import com.netease.nim.uikit.business.session.constant.Extras
import com.netease.nim.uikit.business.uinfo.UserInfoHelper
import com.netease.nim.uikit.common.activity.titlebar.DefaultButtonOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.netease.nim.uikit.impl.database.RedPacketInfo
import com.netease.nimlib.sdk.uinfo.model.NimUserInfo
import kotlinx.android.synthetic.main.fragment_rp_info_c2c.*
import kotlinx.android.synthetic.main.item_red_packet_detail.*
import kotlinx.android.synthetic.main.red_packet_info_layout.*
import java.text.SimpleDateFormat
import java.util.*

class RedPacketInfoC2CFragment : BaseRedPacketInfoFragment() {

    companion object {
        const val EXTRA_FOR_ANIMATION = "EXTRA_FOR_ANIMATION"
        fun newInstance(redPacketInfo: RedPacketInfo, reGetInfo: Boolean = false, showDetailBtn: Boolean = true): RedPacketInfoC2CFragment {

            val bundle = Bundle().apply {
                putSerializable(Extras.EXTRA_RED_PACKET_INFO, redPacketInfo)
                putBoolean(Extras.EXTRA_REGET_RED_PACKET_INFO, reGetInfo)
                putBoolean(Extras.EXTRA_SHOW_BTN, showDetailBtn)
            }
            return RedPacketInfoC2CFragment().apply {
                arguments = bundle
            }
        }

        fun newInstance(redPacketInfo: RedPacketInfo, reGetInfo: Boolean = false, showDetailBtn: Boolean = true , rpInfoFragLifeCallback: RpInfoFragLifeCallback): RedPacketInfoC2CFragment {
            return newInstance(redPacketInfo, reGetInfo, showDetailBtn).apply {
                this.rpInfoFragLifeCallback = rpInfoFragLifeCallback
                arguments?.putBoolean(EXTRA_FOR_ANIMATION, true)
            }
        }
    }

    private val rpInfoPresenter by lazy {
        RPInfoPresenter { _, opened, time, receiverId ->
            fillData(opened, time, receiverId)
        }
    }

    private val showDetailBtn by lazy {
        arguments?.getBoolean(Extras.EXTRA_SHOW_BTN, false) ?: false
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_rp_info_c2c, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        initTitleBar()

        val redPacketInfo = arguments?.getSerializable(Extras.EXTRA_RED_PACKET_INFO) as RedPacketInfo
        val senderUserName = UserInfoHelper.getUserDisplayNameWithNewRule(redPacketInfo.senderId)
        var receiverInfo: NimUserInfo? = null
        redPacketInfo.receiverId?.let {
            receiverInfo = NimUIKit.getUserInfoProvider().getUserInfo(redPacketInfo.receiverId) as NimUserInfo
        }

        if (arguments?.getBoolean(Extras.EXTRA_REGET_RED_PACKET_INFO, false) == true) {
            rpInfoPresenter.getRPInfo(redPacketInfo.packetInfoId, redPacketInfo.senderId)
        }

        //发送方信息
        avatarImageView.loadBuddyAvatar(redPacketInfo.senderId,true)
        userNameView.text = getString(R.string.someone_red_packet, senderUserName)
        redPacketDataView.visibility = if (redPacketInfo.senderId == NimUIKitImpl.getAccount()) View.INVISIBLE else View.VISIBLE
        //红包消息内容
        msgTextView.text = redPacketInfo.packetMessage
        //红包金额
        moneyTextView.text = redPacketInfo.amount
        //红包状态
        redPacketState.visibility = View.VISIBLE
        when {
            redPacketInfo.isOpened -> {
                //状态信息
                redPacketState.text = getString(R.string.red_packet_opened, redPacketInfo.amount)
                receivedList.visibility = View.VISIBLE
                //领取金额
                receiveAmount.text = String.format("%s 元", redPacketInfo.amount)
                //领取时间
                redPacketInfo.time?.let {
                    val format = SimpleDateFormat("MM-dd HH:mm", Locale.CHINA)
                    receiveTime.text = format.format(redPacketInfo.time)
                }
                //领取人信息
                receiverInfo?.let {
                    receiveAvatar.loadBuddyAvatar(receiverInfo?.account)
                    receiveName.text = receiverInfo?.name
                }
            }
            redPacketInfo.isExpired -> {
                redPacketState.text = getString(R.string.red_packet_expired, redPacketInfo.amount)
                receivedList.visibility = View.GONE
                redPacketDataView.visibility = View.INVISIBLE
                expiredTip.visibility = View.VISIBLE
                expiredTip.text = getString(R.string.red_packet_expired_tip)
            }
            else -> {
                redPacketState.text = getString(R.string.red_packet_not_open, redPacketInfo.amount)
                receivedList.visibility = View.GONE
                expiredTip.visibility = View.VISIBLE
                expiredTip.text = getString(R.string.red_packet_not_open_tip)
            }
        }

    }

    override fun onAttach(context: Context?) {
        super.onAttach(context)
        rpInfoPresenter.onAttach(context as FragmentActivity?)
    }

    override fun onDestroy() {
        super.onDestroy()
        rpInfoPresenter.onDetach()
    }

    fun initTitleBar() {
        toolbar.setBackgroundResource(R.drawable.titlebar_red_bg)

        val barOptionsBuilder = TitleBarOptions.Builder()
        if (showDetailBtn) {
            barOptionsBuilder.rightAction(object : DefaultButtonOptions(activity, R.drawable.nim_titlebar_standard_red_btn_selector, R.string.red_packet_list, R.color.white) {
                override fun onClick(view: View?) {
                    C2CRedPacketListActivity.start(activity)
                }
            })
        }

        barOptionsBuilder.leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
            override fun onClick(view: View) {
                activity?.onBackPressed()
            }
        }).titleAction(DefaultTitleOptions(activity, R.string.red_packet, R.color.white))

        toolbar.build(barOptionsBuilder.build())

    }

    private fun fillData(opened: Boolean, time: Long?, receiverId: String?) {
        if (opened) {
            val format = SimpleDateFormat("MM-dd HH:mm", Locale.CHINA)
            receiveTime.text = format.format(time)

            val receiverInfo = NimUIKit.getUserInfoProvider().getUserInfo(receiverId) as NimUserInfo
            receiveAvatar.loadBuddyAvatar(receiverInfo.account)
            receiveName.text = receiverInfo.name
        }
    }

    override fun getRootLayout(): View? = rootLayout

    override fun getAmountText(): View? = moneyTextView

    override fun getAvatarView(): View? = avatarImageView

    override fun isForAnimation(): Boolean = arguments?.getBoolean(EXTRA_FOR_ANIMATION, false) ?: false
}