package com.netease.nim.uikit.business.redpacket.fragment

import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.text.format.DateFormat
import android.text.format.DateUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AbsListView
import android.widget.BaseAdapter
import android.widget.TextView
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.redpacket.ReceivedData
import com.netease.nim.uikit.business.redpacket.activity.C2CRedPacketListActivity
import com.netease.nim.uikit.business.redpacket.getAmount
import com.netease.nim.uikit.business.redpacket.presenter.RPInfoPresenter
import com.netease.nim.uikit.business.session.constant.Extras
import com.netease.nim.uikit.business.uinfo.UserInfoHelper
import com.netease.nim.uikit.common.activity.titlebar.DefaultButtonOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultLeftOptions
import com.netease.nim.uikit.common.activity.titlebar.DefaultTitleOptions
import com.netease.nim.uikit.common.activity.titlebar.TitleBarOptions
import com.netease.nim.uikit.common.util.sys.ScreenUtil
import com.netease.nim.uikit.impl.database.RedPacketInfoV2
import kotlinx.android.synthetic.main.fragment_rp_info.*
import kotlinx.android.synthetic.main.item_red_packet_detail.view.*
import kotlinx.android.synthetic.main.red_packet_info_layout.view.*
import kotlinx.android.synthetic.main.red_packet_received_info_layout.view.*
import java.math.BigDecimal
import java.text.DecimalFormat

class RedPacketInfoFragment : BaseRedPacketInfoFragment(), RPInfoPresenter.RedPacketInfoView {

    companion object {
        const val EXTRA_FOR_ANIMATION = "EXTRA_FOR_ANIMATION"
        fun newInstance(redPacketInfoV2: RedPacketInfoV2, showDetailBtn: Boolean): RedPacketInfoFragment {
            val bundle = Bundle().apply {
                putSerializable(Extras.EXTRA_RED_PACKET, redPacketInfoV2)
                putBoolean(Extras.EXTRA_SHOW_BTN, showDetailBtn)
            }
            return RedPacketInfoFragment().apply {
                arguments = bundle
            }
        }

        fun newInstance(redPacketInfoV2: RedPacketInfoV2, showDetailBtn: Boolean, rpInfoFragLifeCallback: RpInfoFragLifeCallback): RedPacketInfoFragment {
            return newInstance(redPacketInfoV2, showDetailBtn).apply {
                this.rpInfoFragLifeCallback = rpInfoFragLifeCallback
                arguments?.putBoolean(EXTRA_FOR_ANIMATION, true)
            }
        }
    }

    private val showDetailBtn by lazy {
        arguments?.getBoolean(Extras.EXTRA_SHOW_BTN, true) ?: true
    }

    private val rpInfoPresenter by lazy {
        RPInfoPresenter(this)
    }

    private val receivedAdapter by lazy {
        ReceivedListAdapter()
    }

    private val statusInfoView by lazy {
        LayoutInflater.from(activity)
                ?.inflate(R.layout.red_packet_received_info_layout, rootLayout, false)
    }

    private var headView: View? = null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_rp_info, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initTitleBar()

        rpInfoPresenter.onAttach(activity)
        listView.adapter = receivedAdapter
        rpInfoPresenter.extractData(arguments)

    }

    fun initTitleBar() {
        toolbar.setBackgroundResource(R.drawable.titlebar_red_bg)
        val titleOptions = TitleBarOptions.Builder()

        if (showDetailBtn) {
            titleOptions.rightAction(object : DefaultButtonOptions(activity, R.drawable.nim_titlebar_standard_red_btn_selector, R.string.red_packet_list, R.color.white) {
                override fun onClick(view: View?) {
                    C2CRedPacketListActivity.start(activity)
                }
            })
        }

        titleOptions
                .leftAction(object : DefaultLeftOptions(R.drawable.standard_back_red_selector) {
                    override fun onClick(view: View) {
                        activity?.onBackPressed()
                    }
                })
                .titleAction(DefaultTitleOptions(activity, R.string.red_packet, R.color.white))
                .build()

        toolbar.build(titleOptions.build())
    }

    override fun onDestroyView() {
        super.onDestroyView()
        rpInfoPresenter.onDetach()
    }

    override fun onLoadedRedPacketInfo(redPacketInfo: RedPacketInfoV2?) {
        createHeader(redPacketInfo)
        createFooter(redPacketInfo)
        createStatusInfo(redPacketInfo)

        if(!isForAnimation()){
            rpInfoPresenter.loadReceivedList()
        }
    }

    override fun onLoadedReceivedList(receivedList: MutableList<ReceivedData>?) {

        receivedAdapter.addDatas(receivedList)
        receivedAdapter.notifyDataSetChanged()
    }

    override fun refreshStatus(redPacketInfo: RedPacketInfoV2?, receivedList: MutableList<ReceivedData>?) {
        redPacketInfo ?: return
        receivedList ?: return

        var amountSum = BigDecimal(0)
        receivedList.forEach {
            amountSum += it.getAmount()
        }

        redPacketInfo.apply {
            fillStatusInfo(receivedList.count(), amountSum.toString(), redPacketInfo)
        }
    }


    private fun createHeader(redPacketInfo: RedPacketInfoV2?) {
        redPacketInfo ?: return
        val infoView = LayoutInflater.from(activity).inflate(R.layout.red_packet_info_layout, rootLayout, false)
        redPacketInfo.apply {
            infoView.apply {
                avatarImageView.loadBuddyAvatar(redPacketInfo.senderId,true)
                userNameView.text = getString(R.string.someone_red_packet, UserInfoHelper.getUserDisplayName(redPacketInfo.senderId))

                when (originType) {
                    RedPacketTypeV2.C2C -> {
                    }
                    RedPacketTypeV2.GROUP_LUCKY -> {
                        typeIconImageView.visibility = View.VISIBLE
                        typeIconImageView.setImageResource(R.drawable.icon_pin)
                    }

                    RedPacketTypeV2.GROUP_AVG -> {
                        typeIconImageView.visibility = View.VISIBLE
                        typeIconImageView.setImageResource(R.drawable.icon_jun)
                    }
                    else -> {
                    }
                }

                val receivedAmount = when (originType) {
                    RedPacketTypeV2.C2C -> amount
                    RedPacketTypeV2.GROUP_LUCKY,
                    RedPacketTypeV2.GROUP_AVG -> grabAmount
                    else -> amount
                }

                if (isReceived && receivedAmount.toDoubleOrNull() != null) {
                    redPacketDataView.visibility = View.VISIBLE
                    moneyTextView.visibility = View.VISIBLE
                    yuanView.visibility = View.VISIBLE
                    blankView.visibility = View.GONE
                    moneyTextView.text = DecimalFormat("0.00").format(receivedAmount.toDouble())
                } else {
                    redPacketDataView.visibility = View.GONE
                    moneyTextView.visibility = View.GONE
                    yuanView.visibility = View.GONE
                    blankView.visibility = View.VISIBLE
                }

                msgTextView.text = redPacketInfo.message

            }
        }

        headView = infoView
        listView.addHeaderView(infoView)

    }

    private fun createStatusInfo(redPacketInfo: RedPacketInfoV2?) {
        redPacketInfo ?: return

        redPacketInfo.apply {
            fillStatusInfo(receivedCount, receivedAmount, redPacketInfo)
        }

        listView.addHeaderView(statusInfoView)
    }

    private fun fillStatusInfo(receivedCount: Int, receivedAmount: String, redPacketInfo: RedPacketInfoV2?) {
        redPacketInfo?.apply {
            statusInfoView?.redPacketState?.text = when (originStatus) {
                RedPacketStatus.CREATE -> {

                    if (TextUtils.equals(redPacketInfo.senderId, NimUIKit.getAccount())) {
                        getString(R.string.red_packet_received_status_with_amount, "$receivedCount/$count", "$receivedAmount/$amount")
                    } else {
                        getString(R.string.red_packet_received_status, "$receivedCount/$count")
                    }
                }

                RedPacketStatus.FINISH -> {
                    getString(R.string.red_packet_received_status_with_finish, count.toString(), amount, formatTime(lastTime - createTime))
                }

                RedPacketStatus.EXPIRED -> {
                    getString(R.string.red_packet_received_status_with_expired, "$receivedCount/$count", "$receivedAmount/$amount")
                }

                else -> ""
            }

        }
    }

    private fun createFooter(redPacketInfo: RedPacketInfoV2?) {
        if (redPacketInfo != null
                && TextUtils.equals(redPacketInfo.senderId, NimUIKit.getAccount())
                && (redPacketInfo.isGoing || redPacketInfo.isExpired)) {

            val footerView = TextView(context).apply {

                layoutParams = AbsListView.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT).apply {
                    setPadding(0, ScreenUtil.dip2px(20f), 0, ScreenUtil.dip2px(20f))
                }

                setTextSize(TypedValue.COMPLEX_UNIT_SP, 10f)
                setText(R.string.red_packet_expired_desc)
                setTextColor(ContextCompat.getColor(context, R.color.status_text_grey))
                setLineSpacing(ScreenUtil.dip2px(3f).toFloat(), 1f)
                gravity = Gravity.CENTER
                setBackgroundColor(ContextCompat.getColor(context, R.color.red_packet_bg_color))
            }

            listView.addFooterView(footerView)
        }
    }

    private fun formatTime(time: Long): String {
        return when {
            time <= DateUtils.SECOND_IN_MILLIS -> getString(R.string.in_second, "1")
            time <= DateUtils.MINUTE_IN_MILLIS -> getString(R.string.in_second, (time / DateUtils.SECOND_IN_MILLIS).toString())
            time <= DateUtils.HOUR_IN_MILLIS -> getString(R.string.in_minute, (time / DateUtils.MINUTE_IN_MILLIS).toString())
            else -> getString(R.string.in_hour, (time / DateUtils.HOUR_IN_MILLIS).toString())
        }
    }


    class ReceivedListAdapter : BaseAdapter() {

        private val receivedList: MutableList<ReceivedData> = mutableListOf()

        override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
            var itemHolder = convertView?.tag as? ReceivedItemHolder
            if (itemHolder == null) {

                itemHolder = ReceivedItemHolder(LayoutInflater.from(parent?.context).inflate(R.layout.item_red_packet_detail, parent, false))
            }

            itemHolder.fillView(getItem(position))

            itemHolder.itemView.tag = itemHolder

            return itemHolder.itemView
        }

        override fun getItem(position: Int): ReceivedData {
            return receivedList[position]
        }

        override fun getItemId(position: Int): Long = position.toLong()

        override fun getCount(): Int = receivedList.count()


        fun addDatas(dataList: MutableList<ReceivedData>?) {
            dataList?.let {
                receivedList.addAll(dataList)
            }
        }

    }

    class ReceivedItemHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        private val simpleStatusColor = ContextCompat.getColor(itemView.context, R.color.color_gray_alpha40)
        private val winnerStatusColor = ContextCompat.getColor(itemView.context, R.color.red_packet_winner_color)
        private val loserStatusColor = ContextCompat.getColor(itemView.context, R.color.red_packet_loser_color)
        fun fillView(receivedData: ReceivedData) {
            receivedData.apply {
                itemView.apply {
                    receiveName.text = originData.receiveName
                    receiveTime.text = DateFormat.format("MM-dd HH:mm", originData.createTime)
                    receiveAvatar.loadBuddyAvatar(originData.accid)
                    receiveAmount.text = itemView.context.getString(R.string.yuan, getFormatAmount())
                    receiveStatus.apply {
                        when {
                            isWinner -> {
                                visibility = View.VISIBLE
                                setTextColor(winnerStatusColor)
                                setText(R.string.red_packet_winner)
                                setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.icon_best, 0, 0, 0)
                                compoundDrawablePadding = ScreenUtil.dip2px(5f)
                            }
                            isLoser -> {
                                visibility = View.VISIBLE
                                setTextColor(loserStatusColor)
                                setText(R.string.red_packet_loser)
                                setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.icon_worst, 0, 0, 0)
                                compoundDrawablePadding = ScreenUtil.dip2px(5f)
                            }
                            else -> {
                                visibility = View.GONE
                                setTextColor(simpleStatusColor)
                                setCompoundDrawablesRelativeWithIntrinsicBounds(0, 0, 0, 0)
                                compoundDrawablePadding = 0
                            }
                        }
                    }
                }
            }
        }
    }

    override fun isForAnimation(): Boolean = arguments?.getBoolean(EXTRA_FOR_ANIMATION, false) ?: false

    override fun getRootLayout(): View? = rootLayout

    override fun getAmountText(): View? = headView?.findViewById(R.id.moneyTextView)

    override fun getAvatarView(): View? = headView?.findViewById(R.id.avatarImageView)

}
