package com.netease.nim.uikit.business.redpacket.presenter

import android.graphics.Rect
import android.widget.Toast
import com.bullet.libcommonutil.util.UIHandler
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.LoadingAnimDuration
import com.netease.nim.uikit.business.redpacket.RedPacketDialog
import com.netease.nim.uikit.business.redpacket.RemoveFakeViewDelay
import com.netease.nim.uikit.business.redpacket.activity.RedPacketInfoB2CActivity
import com.netease.nim.uikit.business.redpacket.fragment.BaseRedPacketInfoFragment
import com.netease.nim.uikit.business.redpacket.fragment.RedPacketInfoB2CFragment
import com.netease.nim.uikit.business.redpacket.request.RPB2CReqSender
import com.netease.nim.uikit.business.session.extension.B2CRedPacketAttachment
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.netease.nimlib.sdk.msg.model.IMMessage
import com.smartisan.libstyle.toast.NimToast
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.util.concurrent.TimeUnit

class B2CPagerCutter(openRedPacketView: OpenRedPacketView) : PagerCutter(openRedPacketView) {

    private var redPacketDialog: RedPacketDialog? = null

    override fun openRedPacket(imMessage: IMMessage, redPacketAttachment: RedPacketAttachment) {
        if (redPacketAttachment !is B2CRedPacketAttachment) {
            NimToast.makeToast(activity, R.string.not_support_red_packet_type, Toast.LENGTH_LONG).show()
            return
        }

        loading(true)
        RPB2CReqSender.getInviteRedPacketInfo(redPacketAttachment.redPacketId)
                .subscribe(Consumer {
                    loading(false)
                    when (it.status) {
                        1, 2 -> showRpDialog(imMessage, redPacketAttachment)
                        else -> RedPacketInfoB2CActivity.starter(activity, it.amount, redPacketAttachment.isInviter)
                    }
                }, object : RxGrpcErrorHandler(activity) {
                    override fun error(code: Int, errorMsg: String?) {
                        super.error(code, errorMsg)
                        loading(false)
                    }
                })?.let { collectDisposable(it) }
    }

    private fun showRpDialog(imMessage: IMMessage, redPacketAttachment: B2CRedPacketAttachment) {
        val builder = RedPacketDialog.Builder(activity).apply {
            message = redPacketAttachment.message
            open = { open(imMessage, redPacketAttachment) }
            redPacketStatus = RedPacketStatus.CREATE
//                type = RedPacketAttachment.RedPacketType.B2CReceived
        }
        redPacketDialog = builder.build()
        redPacketDialog?.show()
    }

    private fun open(imMessage: IMMessage, redPacketAttachment: B2CRedPacketAttachment) {
        redPacketDialog?.loading()
        RPB2CReqSender.getInviteRedPacket(redPacketAttachment)
                .delay(LoadingAnimDuration, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer {
                    RedPacketAttachment.setIsGetDone(imMessage, true)
                    openRedPacketView?.redPacketIsGetDone()
                    openSuccessAnimation(it.amount, redPacketAttachment)
                }, RxGrpcErrorHandler(activity))
                ?.let { collectDisposable(it) }
    }

    private fun openSuccessAnimation(amount: String?, redPacketAttachment: B2CRedPacketAttachment) {
        val realAmount = amount ?: activity.getString(R.string.default_money)
        val frag = RedPacketInfoB2CFragment.newInstance(realAmount, true, redPacketAttachment.isInviter)
        frag.rpInfoFragLifeCallback = object : BaseRedPacketInfoFragment.RpInfoFragLifeCallback() {
            override fun onViewCreated(frag: BaseRedPacketInfoFragment) {
                frag.startAnimation()
            }

            override fun onAnimationStart(frag: BaseRedPacketInfoFragment, avatarRect: Rect) {
                redPacketDialog?.opening(avatarRect)
            }

            override fun onAnimationEnd(frag: BaseRedPacketInfoFragment) {

                redPacketDialog?.dismiss()
                RedPacketInfoB2CActivity.starter(activity, realAmount, redPacketAttachment.isInviter)
                activity.overridePendingTransition(R.anim.fake_anim, R.anim.fake_anim)
                UIHandler.postDelayed({
                    activity?.supportFragmentManager?.beginTransaction()
                            ?.remove(frag)
                            ?.commitAllowingStateLoss()
                }, RemoveFakeViewDelay)
            }
        }

        activity.supportFragmentManager.beginTransaction()
                .add(android.R.id.content, frag)
                .commitAllowingStateLoss()
    }

    override fun goToDetail(fromAccount: String, redPacketId: String, redPacketTypeV2: RedPacketTypeV2) {
    }
}