package com.netease.nim.uikit.business.redpacket.presenter

import android.text.TextUtils
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.activity.GetCashActivity
import com.netease.nim.uikit.business.redpacket.request.WalletReqSender
import com.netease.nim.uikit.common.presenter.HandleStatusPresenter
import io.reactivex.functions.Consumer
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler

class GetCashPresenter(private val getCashView: GetCashView) : HandleStatusPresenter() {

    private val walletReqSender = WalletReqSender()

    fun initCashList() {
        walletReqSender.getWithdrawList()
                .subscribe(Consumer {
                    val resultList = it.withdrawListOrBuilderList.mapIndexed { _, result ->
                        GetCashActivity.AmountModel(result.amount, result.drawRule, result.canDraw, false)
                    }
                    getCashView.loadedCashList(resultList)
                }, RxGrpcErrorHandler(activity))
                ?.let { collectDisposable(it) }
    }

    fun withdraw(amount: String?) {
        if (amount.isNullOrEmpty() || TextUtils.equals(amount, activity.getString(R.string.default_money))) {
            return
        }

        loading(true)
        walletReqSender.withdraw(amount!!)
                .subscribe(Consumer {
                    loading(false)
                    getCashView.getCashSuccess(it.balance)
                }, object : RxGrpcErrorHandler(activity) {
                    override fun error(code: Int, errorMsg: String?) {
                        super.error(code, errorMsg)
                        loading(false)
                    }
                })
                ?.let { collectDisposable(it) }
    }

    interface GetCashView {
        fun loadedCashList(amountList: List<GetCashActivity.AmountModel>)
        fun getCashSuccess(amount: String?)
    }
}