package com.netease.nim.uikit.business.redpacket.presenter

import android.support.v4.app.FragmentActivity
import android.widget.Toast
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.alipay.AliPay
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.netease.nim.uikit.common.presenter.HandleDisposablePresenter
import com.smartisan.libstyle.toast.NimToast
import com.netease.nimlib.sdk.NIMClient
import com.netease.nimlib.sdk.msg.MsgService
import com.netease.nimlib.sdk.msg.model.IMMessage
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog

class OpenRedPacketPresenter(openRedPacketView: PagerCutter.OpenRedPacketView) : HandleDisposablePresenter() {

    companion object {
        private val TAG = OpenRedPacketPresenter::class.java.simpleName
    }

    private val p2pPagerCutter = P2PPaperCutter(openRedPacketView)
    private val teamPagerCutter = TeamPaperCutter(openRedPacketView)
    private val b2CPagerCutter = B2CPagerCutter(openRedPacketView)

    override fun onAttach(activity: FragmentActivity?) {
        p2pPagerCutter.onAttach(activity)
        teamPagerCutter.onAttach(activity)
        b2CPagerCutter.onAttach(activity)
        super.onAttach(activity)
    }

    override fun onDetach() {
        p2pPagerCutter.onDetach()
        teamPagerCutter.onDetach()
        b2CPagerCutter.onDetach()
        super.onDetach()
    }

    /**
     * IMMessage 的uuid 查询IMMessage 并执行开红包流程
     */
    fun goToDetail(imMessageId: String?) {

        if (activity == null) {
            NimLog.i(TAG, "open red packet failed ! Context is empty !")
            return
        }

        if (imMessageId.isNullOrEmpty()) {
            NimLog.i(TAG, "open red packet failed ! UUID or context is empty !")
            NimToast.makeToast(activity, "查看红包失败", Toast.LENGTH_SHORT).show()
            return
        }

        collectDisposable {
            Observable.fromCallable {
                NIMClient.getService(MsgService::class.java)
                        .queryMessageListByUuidBlock(mutableListOf(imMessageId))
                        .firstOrNull()
            }.subscribeOn(Schedulers.newThread())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({ imMessage ->
                        if (imMessage == null) {
                            NimToast.makeToast(activity, "查看红包失败", Toast.LENGTH_SHORT).show()
                        } else {
                            goToDetail(imMessage)
                        }
                    }, {
                        NimToast.makeToast(activity, "查看红包失败", Toast.LENGTH_SHORT).show()
                    })
        }

    }

    /**
     * 行开红包流程
     */
    fun openRedPacket(imMessage: IMMessage) {

        val attachment = imMessage.attachment
        if (attachment !is RedPacketAttachment) {
            //暂时只支持p2p红包
            NimToast.makeToast(activity, R.string.not_support_red_packet_type, Toast.LENGTH_LONG).show()
            return
        }

        when (attachment.redPacketType) {
            RedPacketAttachment.RedPacketType.Personal -> {
                p2pPagerCutter.openRedPacket(imMessage, imMessage.attachment as RedPacketAttachment)
            }
            RedPacketAttachment.RedPacketType.TeamAVG,
            RedPacketAttachment.RedPacketType.TeamLucky -> {
                teamPagerCutter.openRedPacket(imMessage, imMessage.attachment as RedPacketAttachment)
            }
            RedPacketAttachment.RedPacketType.B2CInviter,
            RedPacketAttachment.RedPacketType.B2CInvitee -> {
                b2CPagerCutter.openRedPacket(imMessage, imMessage.attachment as RedPacketAttachment)
            }
            else -> NimToast.makeToast(activity, R.string.not_support_red_packet_type, Toast.LENGTH_LONG).show()
        }
    }

    fun goToDetail(imMessage: IMMessage) {
        val attachment = imMessage.attachment
        if (attachment !is RedPacketAttachment) {
            //暂时只支持p2p红包
            NimToast.makeToast(activity, R.string.not_support_red_packet_type, Toast.LENGTH_LONG).show()
            return
        }

        when (attachment.redPacketType) {
            RedPacketAttachment.RedPacketType.Personal -> {
                p2pPagerCutter.openRedPacket(imMessage, imMessage.attachment as RedPacketAttachment)
            }

            RedPacketAttachment.RedPacketType.TeamAVG,
            RedPacketAttachment.RedPacketType.TeamLucky -> {
                val redPacketAttachment = imMessage.attachment as RedPacketAttachment
                teamPagerCutter.goToDetail(imMessage.fromAccount, redPacketAttachment.redPacketId, RedPacketTypeV2.forNumber(redPacketAttachment.redPacketType.ordinal))
            }

            else -> NimToast.makeToast(activity, R.string.not_support_red_packet_type, Toast.LENGTH_LONG).show()
        }
    }

}