package com.netease.nim.uikit.business.redpacket.presenter

import android.graphics.Rect
import android.text.TextUtils
import android.widget.Toast
import com.bullet.chat.grpc.ErrorCode
import com.bullet.redpacket.c2c.grpc.GetRedPacketInfoRequest
import com.bullet.redpacket.c2c.grpc.OpenRedPacketRequest
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.redpacket.LoadingAnimDuration
import com.netease.nim.uikit.business.redpacket.RedPacketDialog
import com.netease.nim.uikit.business.redpacket.RemoveFakeViewDelay
import com.netease.nim.uikit.business.redpacket.request.RPC2CReqSender
import com.netease.nim.uikit.business.redpacket.activity.RedPacketInfoC2CActivity
import com.netease.nim.uikit.business.redpacket.fragment.BaseRedPacketInfoFragment
import com.netease.nim.uikit.business.redpacket.fragment.RedPacketInfoC2CFragment
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.bullet.libcommonutil.util.UIHandler
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.impl.database.RedPacketInfo
import com.netease.nimlib.sdk.msg.model.IMMessage
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.ServerCodeErrorUtil
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.lang.Exception
import java.util.concurrent.TimeUnit

class P2PPaperCutter(openRedPacketView: OpenRedPacketView?) : PagerCutter(openRedPacketView) {

    private val rpC2CReqSender = RPC2CReqSender()
    private var rpDialog: RedPacketDialog? = null
    override fun openRedPacket(imMessage: IMMessage, redPacketAttachment: RedPacketAttachment) {
        //先请求红包详情
        val request = GetRedPacketInfoRequest.newBuilder()
                .setRedPacketId(redPacketAttachment.redPacketId)
                .setFromAccid(imMessage.fromAccount)
                .build()

        loading(true)
        collectDisposable {
            rpC2CReqSender.getRedPacketInfo(request)
                    .subscribe(Consumer { redPacketInfo ->
                        updateRedPacketStatus(redPacketInfo.originStatus, imMessage, redPacketAttachment)
                        gotoRedPacketDetail(imMessage, redPacketAttachment, redPacketInfo)
                        loading(false)
                    }, object : RxGrpcErrorHandler() {
                        override fun error(code: Int, errorMsg: String) {
                            NimToast.makeToast(activity, errorMsg, Toast.LENGTH_SHORT).show()
                            loading(false)
                        }
                    })
        }
    }

    override fun goToDetail(fromAccount: String, redPacketId: String, redPacketTypeV2: RedPacketTypeV2) {
    }

    private fun gotoRedPacketDetail(imMessage: IMMessage, attachment: RedPacketAttachment, packetInfo: RedPacketInfo) {
        //自己发的直接进入详情页
        //or 已领取，直接进入详情页
        if (TextUtils.equals(NimUIKit.getAccount(), packetInfo.senderId)
                || attachment.isGetDone) {
            RedPacketInfoC2CActivity.starter(activity, packetInfo)
            return
        }


        rpDialog = RedPacketDialog.Builder(activity)
                .apply {
                    fromAccount = packetInfo.senderId
                    message = attachment.message
                    redPacketStatus = if (attachment.isExpired) RedPacketStatus.EXPIRED else RedPacketStatus.CREATE
                    open = {
                        sendOpenRedPacketRequest(imMessage, attachment, packetInfo)
                    }
                    goToDetail = { RedPacketInfoC2CActivity.starter(activity, packetInfo) }
                }.build()
        rpDialog?.show()
    }

    /**
     * 发送开红包请求
     */
    private fun sendOpenRedPacketRequest(imMessage: IMMessage, attachment: RedPacketAttachment, packetInfo: RedPacketInfo) {
        val request = OpenRedPacketRequest.newBuilder()
                .setRedPacketId(attachment.redPacketId)
                .setFromAccid(packetInfo.senderId)
                .setOriginMessageId(imMessage.uuid)
                .build()

        rpDialog?.loading()
        rpC2CReqSender.openRedPacket(request)
                .delay(LoadingAnimDuration, TimeUnit.MILLISECONDS)
                .retryWhen {
                    it.flatMap {
                        if (it is Exception && ServerCodeErrorUtil.getErrorCode(it) == ErrorCode.ERROR_NOT_BIND_ALIPAY_VALUE) {
                            auth{
                                rpDialog?.dismiss()
                            }
                        } else {
                            Observable.error(it)
                        }
                    }
                }
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer {
                    notifyRedPacketGetDone(imMessage, true)

                    val infoFrag = RedPacketInfoC2CFragment.newInstance(packetInfo, true, true, object: BaseRedPacketInfoFragment.RpInfoFragLifeCallback(){
                        override fun onViewCreated(frag: BaseRedPacketInfoFragment) {
                            frag.startAnimation()
                        }

                        override fun onAnimationStart(frag: BaseRedPacketInfoFragment, avatarRect: Rect) {
                            rpDialog?.opening(avatarRect)
                        }

                        override fun onAnimationEnd(frag: BaseRedPacketInfoFragment) {

                            packetInfo.isOpened = true
                            RedPacketInfoC2CActivity.starter(activity, packetInfo, true)
                            activity.overridePendingTransition(0, 0)

                            rpDialog?.dismiss()
                            UIHandler.postDelayed({
                                activity.supportFragmentManager
                                        .beginTransaction()
                                        .remove(frag)
                                        .commitNowAllowingStateLoss()
                            }, RemoveFakeViewDelay)
                        }

                    })

                    activity.supportFragmentManager
                            .beginTransaction()
                            .add(android.R.id.content, infoFrag)
                            .commitNowAllowingStateLoss()

                }, object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String?) {
                        loading(false)
                        NimLog.i(OpenRedPacketPresenter::class.simpleName, "open error:$errorMsg code:$code")
                        when (code) {
                            ErrorCode.ERROR_RED_ENVELOP_RECEIVED_VALUE -> {
                                RedPacketAttachment.setIsGetDone(imMessage, true)
                                RedPacketInfoC2CActivity.starter(activity, packetInfo)
                            }
                            ErrorCode.ERROR_THIRD_PARTY_BIND_OTHER_VALUE -> {
                                rpDialog?.dismiss()
                                showToast(R.string.ali_have_bind_other_account, activity)
                            }
                            else -> {
                                showToast(errorMsg
                                        ?: activity.getString(R.string.get_rp_failed), activity)
                            }
                        }
                    }
                })?.let { collectDisposable(it) }
    }

}