package com.netease.nim.uikit.business.redpacket.presenter

import android.content.Context
import android.widget.Toast
import com.bullet.chat.grpc.ErrorCode
import com.bullet.redpacket.c2c.grpc.GetRedPacketInfoResponse
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.google.protobuf.Empty
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.alipay.AliPay
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.netease.nim.uikit.business.uinfo.UserInfoHelper
import com.netease.nim.uikit.common.presenter.HandleStatusPresenter
import com.smartisan.libstyle.toast.NimToast
import com.netease.nimlib.sdk.msg.model.IMMessage
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler

abstract class PagerCutter(protected var openRedPacketView: OpenRedPacketView?) : HandleStatusPresenter() {
    private val aliPay = AliPay()

    abstract fun openRedPacket(imMessage: IMMessage, redPacketAttachment: RedPacketAttachment)
    abstract fun goToDetail(fromAccount: String, redPacketId: String, redPacketTypeV2: RedPacketTypeV2)

    override fun onDetach() {
        openRedPacketView = null
        super.onDetach()
    }

    protected fun auth(errorCallback: (() -> Unit)? = null): Observable<Empty> {
        return aliPay.startAuthRequest(activity)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .doOnError(object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String) {
                        loading(false)
                        when (code) {
                            ErrorCode.ERROR_THIRD_PARTY_IS_BIND_VALUE -> showToast(R.string.ali_have_bind_other_account, activity)
                            else -> showToast(R.string.bind_failed, activity)
                        }
                    }

                    override fun error(throwable: Throwable) {
                        errorCallback?.invoke()
                        loading(false)
                        showToast(throwable.message, activity)
                    }
                })
                .doOnNext {
                    showToast("绑定成功", activity)
                }

    }

    /**
     * 更新本地红包状态
     */
    fun updateRedPacketStatus(status: GetRedPacketInfoResponse.Status, imMessage: IMMessage, redPacketAttachment: RedPacketAttachment) {
        if (status == GetRedPacketInfoResponse.Status.RECEIVED && !redPacketAttachment.isGetDone) {
            notifyRedPacketGetDone(imMessage, true)
        } else if (status == GetRedPacketInfoResponse.Status.RECALL && !redPacketAttachment.isExpired) {
            notifyRedPacketExpired(imMessage, true)
        }

    }

    fun notifyRedPacketGetDone(imMessage: IMMessage, isGetDone: Boolean) {
        RedPacketAttachment.setIsGetDone(imMessage, isGetDone)
        openRedPacketView?.redPacketIsGetDone()

    }

    fun notifyRedPacketExpired(imMessage: IMMessage, isExpired: Boolean) {
        RedPacketAttachment.setIsExpired(imMessage, isExpired)
        openRedPacketView?.redPacketIsGetDone()
    }

    fun getSenderName(senderId: String): String {
        val currentUserName = UserInfoHelper.getUserDisplayNameWithNewRule(senderId)
        return currentUserName ?: "你的好友"
    }

    fun showToast(msg: String?, context: Context) {
        msg ?: return
        NimToast.makeToast(context, msg, Toast.LENGTH_SHORT).show()
    }

    fun showToast(msgId: Int?, context: Context) {
        msgId ?: return
        NimToast.makeToast(context, msgId, Toast.LENGTH_SHORT).show()
    }

    abstract class OpenRedPacketView {
        abstract fun redPacketIsGetDone()
    }
}