package com.netease.nim.uikit.business.redpacket.presenter

import android.os.Bundle
import android.widget.Toast
import com.bullet.redpacket.c2c.grpc.GetRedPacketInfoRequest
import com.bullet.redpacket.c2c.grpc.GetRedPacketInfoResponse
import com.bullet.redpacket.c2c.v2.grpc.GetReceivedListRequest
import com.bullet.redpacket.c2c.v2.grpc.GetReceivedListResponse
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.redpacket.ReceivedData
import com.netease.nim.uikit.business.redpacket.getAmount
import com.netease.nim.uikit.business.redpacket.getCreateTime
import com.netease.nim.uikit.business.session.constant.Extras.EXTRA_RED_PACKET
import com.netease.nim.uikit.common.presenter.HandleDisposablePresenter
import com.smartisan.libstyle.toast.NimToast
import com.netease.nim.uikit.impl.database.FlashAppDBHelper
import com.netease.nim.uikit.impl.database.RedPacketInfoV2
import com.netease.nim.uikit.impl.database.RedPacketReceived
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient

import smartisan.cloud.im.grpc.ProtoBuffManager
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog
import com.bullet.point.grpc.B2CRedPacketInfoRequest
import com.bullet.point.grpc.B2CRedPacketInfoResponse
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import smartisan.cloud.im.rxjava.RxGrpcResultHandler

class RPInfoPresenter : HandleDisposablePresenter {
    val tag = RPInfoPresenter::class.java.simpleName!!
    var callback: ((String, Boolean, Long?, String?) -> Unit)? = null

    private var redPacketInfoView: RedPacketInfoView? = null

    var redPacketInfo: RedPacketInfoV2? = null

    constructor(callback: ((String, Boolean, Long?, String?) -> Unit)?) : super() {
        this.callback = callback
    }

    constructor(redPacketInfoView: RedPacketInfoView?) : super() {
        this.redPacketInfoView = redPacketInfoView
    }

    override fun extractData(bundle: Bundle?) {
        redPacketInfo = bundle?.getSerializable(EXTRA_RED_PACKET) as RedPacketInfoV2
        redPacketInfoView?.onLoadedRedPacketInfo(redPacketInfo)
    }

    fun loadReceivedList() {
        val rpInfo = redPacketInfo
        if (rpInfo == null || rpInfo.redPacketId.isNullOrEmpty()) {
            NimLog.i(tag, "load list failed!! because red packet id is null!!!")
            NimToast.makeToast(activity, R.string.load_data_failed, Toast.LENGTH_LONG).show()
            return
        }

        Observable.fromCallable {
            val redPacketDao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
            val rpReceived = redPacketDao.queryRPReceived(rpInfo.redPacketId).firstOrNull()
            val result: List<ReceivedData>
            if (rpReceived == null || rpReceived.listJson.isNullOrEmpty()) {
                val request = GetReceivedListRequest.newBuilder()
                        .setPageNo(0)
                        .setPageSize(rpInfo.count)
                        .setRedPacketId(rpInfo.redPacketId)
                        .setFromAccid(rpInfo.senderId)
                        .build()
                val response = ProtoBuffManager.getInstance().doRedPacketStubV2().getReceivedList(request)
                result = transformData(response.receivedListList)

                if (!rpInfo.isGoing) {
                    redPacketDao.insertRPReceived(RedPacketReceived(rpInfo.redPacketId, Gson().toJson(result)))
                }
            } else {
                result = Gson().fromJson(rpReceived.listJson, object : TypeToken<MutableList<ReceivedData>>() {}.type)
            }
            sortByAmount(result)
            result?.sortedByDescending { it.getCreateTime() }?.toMutableList()
        }.subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer {
                    if (it != null && !it.isEmpty()) {
                        redPacketInfoView?.onLoadedReceivedList(it)
                        redPacketInfoView?.refreshStatus(redPacketInfo, it)
                    }
                }, object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String?) {
                        NimToast.makeToast(activity, errorMsg ?: "error", Toast.LENGTH_LONG).show()
                    }
                })?.let { collectDisposable(it) }

    }

    private fun transformData(originData: List<GetReceivedListResponse.ReceivedList>): List<ReceivedData> {
        if (originData.isEmpty()) {
            return listOf()
        }
        return originData.map {
            ReceivedData(it, false, false)
        }
    }

    private fun sortByAmount(receivedDatas: List<ReceivedData>) {
        if (receivedDatas.isEmpty()) {
            return
        }

        var winnerTempData: ReceivedData? = null
        var loserTempData: ReceivedData? = null
        receivedDatas.forEach { receivedData ->

            // 拼手气红包要筛选出手气最佳
            if (redPacketInfo?.originType == RedPacketTypeV2.GROUP_LUCKY && redPacketInfo?.isFinish == true) {
                if (receivedData.getAmount() > winnerTempData.getAmount()
                        || (receivedData.getAmount() == winnerTempData.getAmount() && receivedData.getCreateTime() < winnerTempData.getCreateTime())) {
                    winnerTempData = receivedData
                }

                if (loserTempData == null) {
                    loserTempData = receivedData
                }

                if (receivedData.getAmount() < loserTempData.getAmount()
                        || (receivedData.getAmount() == loserTempData.getAmount() && receivedData.getCreateTime() > loserTempData.getCreateTime())) {
                    loserTempData = receivedData
                }
            }
        }

        if (redPacketInfo?.isFinish == true) {
            winnerTempData?.isWinner = true

            if (loserTempData.getAmount() <= winnerTempData.getAmount()) {
                loserTempData?.isLoser = true
            }

        }
    }


    fun getRPInfo(id: String, senderId: String) {
        val disposable = Observable.create(object : RxGrpcResultHandler<GetRedPacketInfoResponse>() {
            override fun subscribe(): GetRedPacketInfoResponse {
                val request = GetRedPacketInfoRequest.newBuilder()
                        .setRedPacketId(id)
                        .setFromAccid(senderId)
                        .build()

                return ProtoBuffManager.getInstance()
                        .doRedPacketStub()
                        .getRedPacketInfo(request)
            }
        }).subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer<GetRedPacketInfoResponse> { response ->
                    val amount = response.amount
                    val opened = response.receivedCount != 0
                    if (opened) {
                        callback?.invoke(amount, opened, response.receivedRedPacketsOrBuilderList[0].receivedAt, response.receivedRedPacketsOrBuilderList[0].receivedAccid)
                    } else {
                        callback?.invoke(amount, opened, null, null)
                    }

                }, RxGrpcErrorHandler(activity))

        collectDisposable(disposable)
    }

    fun getB2CRPInfo(id: String) {
        val disposable = Observable.create(object : RxGrpcResultHandler<B2CRedPacketInfoResponse>() {
            override fun subscribe(): B2CRedPacketInfoResponse {
                val request = B2CRedPacketInfoRequest.newBuilder()
                        .setRedPacketId(id)
                        .build()

                return ProtoBuffManager.getInstance()
                        .doB2CRedPacketStub()
                        .getInviteRedPacketInfo(request)
            }
        }).subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer<B2CRedPacketInfoResponse> { response ->
                    val amount = response.amount
                    callback?.invoke(amount, true, null, null)

                }, RxGrpcErrorHandler(activity))

        collectDisposable(disposable)
    }

    interface RedPacketInfoView {
        fun onLoadedRedPacketInfo(redPacketInfo: RedPacketInfoV2?)
        fun onLoadedReceivedList(receivedList: MutableList<ReceivedData>?)
        fun refreshStatus(redPacketInfo: RedPacketInfoV2?, receivedList: MutableList<ReceivedData>?)
    }


}