package com.netease.nim.uikit.business.redpacket.presenter

import android.graphics.Rect
import com.bullet.chat.grpc.ErrorCode
import com.bullet.redpacket.c2c.v2.grpc.GetStatusResponse
import com.bullet.redpacket.c2c.v2.grpc.OpenRequest
import com.bullet.redpacket.c2c.v2.grpc.RedPacketStatus
import com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2
import com.google.protobuf.Empty
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.redpacket.LoadingAnimDuration
import com.netease.nim.uikit.business.redpacket.RedPacketDialog
import com.netease.nim.uikit.business.redpacket.RemoveFakeViewDelay
import com.netease.nim.uikit.business.redpacket.activity.RedPacketInfoActivity
import com.netease.nim.uikit.business.redpacket.fragment.BaseRedPacketInfoFragment
import com.netease.nim.uikit.business.redpacket.fragment.RedPacketInfoFragment
import com.netease.nim.uikit.business.redpacket.request.RedPacketReqSender
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.bullet.libcommonutil.util.UIHandler
import com.netease.nimlib.sdk.msg.model.IMMessage
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.functions.Function
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.ServerCodeErrorUtil
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.lang.Exception
import java.util.concurrent.TimeUnit

class TeamPaperCutter(openRedPacketView: OpenRedPacketView?) : PagerCutter(openRedPacketView) {

    private val Tag = TeamPaperCutter::class.java.simpleName
    private val requestSender = RedPacketReqSender()

    private var redPacketDialog: RedPacketDialog? = null
    private var dialogBuilder: RedPacketDialog.Builder? = null

    private var retryAuth = Function<Observable<Throwable>, Observable<Empty>> { it ->
        it.flatMap {
            if (it is Exception && ServerCodeErrorUtil.getErrorCode(it) == ErrorCode.ERROR_NOT_BIND_ALIPAY_VALUE) {
                auth{
                    redPacketDialog?.dismiss()
                }
            } else {
                Observable.error(it)
            }
        }
    }

    override fun openRedPacket(imMessage: IMMessage, redPacketAttachment: RedPacketAttachment) {

        loading(true)
        requestSender.getStatus(imMessage.fromAccount, redPacketAttachment.redPacketId)
                .subscribe(Consumer<GetStatusResponse> {
                    loading(false)
                    if (it.isReceived) {
                        notifyRedPacketGetDone(imMessage, true)
                        goToDetail(imMessage.fromAccount, redPacketAttachment.redPacketId, RedPacketTypeV2.forNumber(redPacketAttachment.redPacketType.ordinal))
                    } else {
                        checkStatus(it.status, imMessage)
                        showRPDialog(it.status, imMessage, redPacketAttachment)
                    }

                }, object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String?) {
                        showToast("$errorMsg", activity)
                        loading(false)
                    }
                })?.let { collectDisposable(it) }

    }

    override fun goToDetail(fromAccount: String, redPacketId: String, redPacketTypeV2: RedPacketTypeV2) {
        loading(true)
        requestSender.getInfo(fromAccount, redPacketId, redPacketTypeV2)
                .subscribe(Consumer {
                    loading(false)
                    redPacketDialog?.dismiss()
                    RedPacketInfoActivity.starter(activity, it, true)
                }, object : RxGrpcErrorHandler() {
                    override fun error(code: Int, errorMsg: String?) {
                        loading(false)
                        showToast("$errorMsg", activity)
                    }
                })?.let { collectDisposable(it) }
    }

    private fun checkStatus(status: RedPacketStatus, imMessage: IMMessage) {
        when (status) {
            RedPacketStatus.CREATE -> {
                // do nothing
            }
            RedPacketStatus.EXPIRED -> {
                notifyRedPacketExpired(imMessage, true)
            }

            RedPacketStatus.FINISH -> {
                notifyRedPacketFinish(imMessage, true)
            }
            else -> {
                showToast(R.string.red_packet_status_error, activity)
                NimLog.e(Tag, "red packet status error:${status.number}")
                return
            }
        }
    }

    private fun showRPDialog(status: RedPacketStatus, imMessage: IMMessage, redPacketAttachment: RedPacketAttachment) {
        dialogBuilder = RedPacketDialog.Builder(activity)
                .apply {
                    fromAccount = imMessage.fromAccount
                    message = redPacketAttachment.message
                    redPacketStatus = status
                    type = redPacketAttachment.redPacketType

                    goToDetail = {
                        goToDetail(imMessage.fromAccount, redPacketAttachment.redPacketId, RedPacketTypeV2.forNumber(redPacketAttachment.redPacketType.ordinal))
                    }

                    open = {
                        sendOpenRequest(
                                imMessage,
                                redPacketAttachment,
                                RedPacketTypeV2.forNumber(redPacketAttachment.redPacketType.ordinal))
                    }
                }
        redPacketDialog = dialogBuilder?.build()
        redPacketDialog?.show()
    }

    private fun sendOpenRequest(imMessage: IMMessage, redPacketAttachment: RedPacketAttachment, type: RedPacketTypeV2) {
        val request = OpenRequest.newBuilder()
                .setSessionId(imMessage.sessionId)
                .setFromAccid(imMessage.fromAccount)
                .setOriginMessageId(imMessage.uuid)
                .setRedPacketId(redPacketAttachment.redPacketId)
                .setRedPacketType(type)
                .build()

//        loading(true)
        redPacketDialog?.loading()

        requestSender.open(request)
                .delay(LoadingAnimDuration,TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .retryWhen(retryAuth)
                .subscribe(Consumer {
                    notifyRedPacketGetDone(imMessage, true)

                    val redPacketInfoV2 = requestSender.createInfo(it, type, true)
                    val redPacketInfoFragment = RedPacketInfoFragment.newInstance(redPacketInfoV2, true, object : BaseRedPacketInfoFragment.RpInfoFragLifeCallback() {
                        override fun onViewCreated(frag: BaseRedPacketInfoFragment) {
                            frag.startAnimation()
                        }
                        override fun onAnimationStart(frag: BaseRedPacketInfoFragment, avatarRect: Rect) {
                            redPacketDialog?.opening(avatarRect)
                        }

                        override fun onAnimationEnd(frag: BaseRedPacketInfoFragment) {
                            redPacketDialog?.dismiss()
                            RedPacketInfoActivity.starter(activity, redPacketInfoV2, true)
                            activity.overridePendingTransition(R.anim.fake_anim, R.anim.fake_anim)
                            UIHandler.postDelayed({
                                activity?.supportFragmentManager?.beginTransaction()
                                        ?.remove(frag)
                                        ?.commitAllowingStateLoss()
                            }, RemoveFakeViewDelay)
                        }
                    })

                    activity.supportFragmentManager.beginTransaction()
                            .add(android.R.id.content, redPacketInfoFragment)
                            .commitAllowingStateLoss()

                }, OpenErrorHandler(imMessage))
                ?.let { collectDisposable(it) }

    }

    private fun notifyRedPacketFinish(imMessage: IMMessage, isFinish: Boolean) {
        RedPacketAttachment.setIsFinish(imMessage, isFinish)
        openRedPacketView?.redPacketIsGetDone()
    }

    inner class OpenErrorHandler(private val imMessage: IMMessage) : RxGrpcErrorHandler() {
        override fun error(code: Int, errorMsg: String?) {
            loading(false)

            when (code) {
                ErrorCode.ERROR_C2G_RED_ENVELOP_FINISHED_VALUE -> {
                    notifyRedPacketFinish(imMessage, true)
                    dialogBuilder?.redPacketStatus = RedPacketStatus.FINISH
                    redPacketDialog?.initView(dialogBuilder)
                }
                ErrorCode.ERROR_THIRD_PARTY_BIND_OTHER_VALUE -> {
                    redPacketDialog?.dismiss()
                    showToast(R.string.ali_have_bind_other_account, activity)
                }
                else -> {
                    showToast(errorMsg
                            ?: activity.getString(R.string.get_rp_failed), activity)
                }
            }
        }
    }

}