package com.netease.nim.uikit.business.redpacket.request

import com.bullet.point.grpc.*
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.session.extension.B2CRedPacketAttachment
import com.netease.nim.uikit.impl.database.FlashAppDBHelper
import com.netease.nim.uikit.impl.database.RedPacketInfoV2
import io.reactivex.Observable
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.rxjava.RxGrpcRequestSender

object RPB2CReqSender : RxGrpcRequestSender<RecruitGrpc.RecruitBlockingStub>() {

    fun createInviteUrl(request: CreateInviteUrlRequest): Observable<CreateInviteUrlResponse> {
        return newRequest { stub -> stub.createInviteUrl(request) }
    }

    fun getInviteRedPacket(rpAttachment: B2CRedPacketAttachment): Observable<GetInviteRedPacketResponse> {
        val request = GetInviteRedPacketRequest.newBuilder().setRedPacketId(rpAttachment.redPacketId).build()
        return newRequestNoScheduler { stub ->
            val response = stub.getInviteRedPacket(request)

            FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao().apply {
                insertRPInfo(RedPacketInfoV2().apply {
                    redPacketId = rpAttachment.redPacketId
                    redPacketType = rpAttachment.redPacketType.ordinal
                    amount = response.amount
                })
            }

            response
        }
    }

    fun getInviteRedPacketInfo(rpId: String): Observable<B2CRedPacketInfoResponse> {
        val request = B2CRedPacketInfoRequest.newBuilder().setRedPacketId(rpId).build()
        return newRequest { stub ->
            val dao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
            val redPacketInfoV2 = dao.queryRPInfo(rpId).firstOrNull()
            if (redPacketInfoV2 == null) {
                stub.getInviteRedPacketInfo(request)
            } else {
                B2CRedPacketInfoResponse.newBuilder().apply {
                    amount = redPacketInfoV2.amount
                    status = 3
                }.build()
            }

        }
    }

    fun checkInviteRedPacket(userId: String): Observable<CheckInviteRedPacketResponse> {
        val request = CheckInviteRedPacketRequest.newBuilder().setUserId(userId).build()
        return newRequest { stub -> stub.checkInviteRedPacket(request) }
    }

    override fun getStub(): Observable<RecruitGrpc.RecruitBlockingStub> {
        return Observable.fromCallable { ProtoBuffManager.getInstance().doB2CRedPacketStub() }
    }
}