package com.netease.nim.uikit.business.redpacket.request

import com.bullet.redpacket.c2c.grpc.*
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.impl.database.FlashAppDBHelper
import com.netease.nim.uikit.impl.database.RedPacketInfo
import io.reactivex.Observable
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.rxjava.RxGrpcRequestSender
import smartisan.cloud.im.rxjava.RxGrpcResultHandler

class RPC2CReqSender : RxGrpcRequestSender<C2CRedPacketGrpc.C2CRedPacketBlockingStub>() {

    fun openRedPacket(request: OpenRedPacketRequest): Observable<OpenRedPacketResponse> {
        return newRequestNoScheduler { stub -> stub.openRedPacket(request) }
    }

    fun getRedPacketInfo(request: GetRedPacketInfoRequest): Observable<RedPacketInfo> {
        return defaultHandleOn(
                Observable.create(object : RxGrpcResultHandler<RedPacketInfo>() {
                    override fun subscribe(): RedPacketInfo {
                        val redPacketDao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
                        var redPacketInfo:RedPacketInfo? = redPacketDao.query(request.redPacketId).firstOrNull()

                        if (redPacketInfo == null) {
                            redPacketInfo = createRedPacketInfo(request,
                                    ProtoBuffManager.getInstance()
                                            .doRedPacketStub()
                                            .getRedPacketInfo(request))

                            if(redPacketInfo.isOpened || redPacketInfo.isExpired){
                                redPacketDao.insert(redPacketInfo)
                            }
                        }

                        return redPacketInfo
                    }
                }))
    }

    private fun createRedPacketInfo(request: GetRedPacketInfoRequest, response: GetRedPacketInfoResponse): RedPacketInfo {
        //开红包逻辑
        val amount = response.amount
        val opened = response.status == GetRedPacketInfoResponse.Status.RECEIVED
        val expired = response.status == GetRedPacketInfoResponse.Status.RECALL
        var time: Long? = null
        var receiverId: String? = null
        if (opened) {
            time = response.receivedRedPacketsOrBuilderList[0].receivedAt
            receiverId = response.receivedRedPacketsOrBuilderList[0].receivedAccid
        }

        return RedPacketInfo(response.redPacketId, response.message, amount, request.fromAccid, receiverId, time, opened, expired, response.status.number)
    }

    fun getRedPacketRecordList(request: GetRedPacketRecordListRequest): Observable<GetRedPacketRecordListResponse> {
        return newRequest { stub -> stub.getRedPacketRecordList(request) }
    }

    override fun getStub(): Observable<C2CRedPacketGrpc.C2CRedPacketBlockingStub> {
        return Observable.fromCallable { ProtoBuffManager.getInstance().doRedPacketStub() }
    }
}
