package com.netease.nim.uikit.business.redpacket.request

import com.bullet.redpacket.c2c.v2.grpc.*
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.impl.database.FlashAppDBHelper
import com.netease.nim.uikit.impl.database.RedPacketInfoV2
import io.reactivex.Observable
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.rxjava.RxGrpcRequestSender

class RedPacketReqSender : RxGrpcRequestSender<C2CRedPacketV2Grpc.C2CRedPacketV2BlockingStub>() {

    fun open(request: OpenRequest): Observable<PacketInfoResult> {
        return newRequestNoScheduler { stub -> stub.open(request) }
    }

    fun getStatus(fromAccount: String, redPacketId: String): Observable<GetStatusResponse> {

        return defaultHandleOn(
                Observable.fromCallable<GetStatusResponse> {
                    val redPacketDao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
                    val rpInfo = redPacketDao.queryRPInfo(redPacketId).firstOrNull()

                    if (rpInfo != null && !rpInfo.isGoing) {
                        GetStatusResponse.newBuilder()
                                .setIsReceived(rpInfo.isReceived)
                                .setStatus(rpInfo.originStatus)
                                .build()

                    } else {
                        val getStatusRequest = GetStatusRequest.newBuilder()
                                .setFromAccid(fromAccount)
                                .setRedPacketId(redPacketId)
                                .build()
                        val response = ProtoBuffManager.getInstance().doRedPacketStubV2().getStatus(getStatusRequest)
                        val redPacketInfoV2 = createInfo(response.status, redPacketId, fromAccount, response.isReceived)
                        if (!redPacketInfoV2.isGoing) {
                            redPacketDao.insertRPInfo(redPacketInfoV2)
                        }
                        response
                    }

                })
    }

    fun getInfo(fromAccount: String, redPacketId: String, redPacketTypeV2: RedPacketTypeV2): Observable<RedPacketInfoV2> {

        return defaultHandleOn(Observable.fromCallable<RedPacketInfoV2> {
            val redPacketDao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
            var rpInfo = redPacketDao.queryRPInfo(redPacketId).firstOrNull()

            if (rpInfo == null || rpInfo.isGoing || !rpInfo.isComplete) {
                val getStatusRequest = GetInfoRequest.newBuilder()
                        .setFromAccid(fromAccount)
                        .setRedPacketId(redPacketId)
                        .build()

                val packetInfoResult = ProtoBuffManager.getInstance().doRedPacketStubV2().getInfo(getStatusRequest)
                rpInfo = createInfo(packetInfoResult, redPacketTypeV2, packetInfoResult.isReceived)

                if (!rpInfo.isGoing) {
                    redPacketDao.insertRPInfo(rpInfo)
                }
            }

            rpInfo
        })
    }

    fun createInfo(result: PacketInfoResult, redPacketType: RedPacketTypeV2, isGetDone: Boolean): RedPacketInfoV2 {
        return RedPacketInfoV2(result.redPacketId, result.amount, result.count, result.receivedCount, result.receivedAmount, result.createTime, result.lastTime, result.message, result.statusValue, result.fromAccid, isGetDone, result.grabAmount, redPacketType.ordinal)
    }

    private fun createInfo(status: RedPacketStatus, redPacketId: String, fromAccount: String, isGetDone: Boolean): RedPacketInfoV2 {
        return RedPacketInfoV2(redPacketId, "", 0, 0, "", 0, 0, "", status.ordinal, fromAccount, isGetDone, "", RedPacketTypeV2.UNKNOWN_VALUE)
    }

    fun getRedPacketRecordList(request: GetRecordListV2Request): Observable<GetRecordListV2Response> {
        return newRequest { stub -> stub.getRecordListV2(request) }
    }

    /**
     * 订单创建成功将红吧id 发送回给服务器
     */
    fun payNotifyRequest(redPacketId: String): Observable<PayNotifyResponse> {
        val request = PayNotifyRequest.newBuilder()
                .setRedPacketId(redPacketId)
                .build()
        return newRequest{ stub ->
            stub.payNotify(request)
        }
    }

    override fun getStub(): Observable<C2CRedPacketV2Grpc.C2CRedPacketV2BlockingStub> {
        return Observable.fromCallable { ProtoBuffManager.getInstance().doRedPacketStubV2() }
    }

}
