package com.netease.nim.uikit.business.redpacket.viewholder

import android.annotation.SuppressLint
import android.content.Intent
import android.graphics.Color
import android.net.Uri
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.Toast
import com.bullet.chat.grpc.ErrorCode.ERROR_AREADY_IS_ZD_USER_VALUE
import com.bullet.messenger.contact.provider.ContactManager
import com.bullet.point.grpc.CreateInviteUrlRequest
import com.bullet.point.grpc.InviteSource
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.contact.core.item.ContactItem
import com.netease.nim.uikit.business.contact.core.model.ContactDataAdapter
import com.netease.nim.uikit.business.contact.core.viewholder.AbsContactViewHolder
import com.netease.nim.uikit.business.redpacket.request.RPB2CReqSender
import com.smartisan.libstyle.toast.NimToast
import io.reactivex.functions.Consumer
import kotlinx.android.synthetic.main.check_contact_view_holder.view.*
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.util.ArrayList

class CheckContactHolder : AbsContactViewHolder<ContactItem>() {

    override fun refresh(adapter: ContactDataAdapter?, position: Int, item: ContactItem?) {
        item ?: return
        val contact = item.contact
        view.apply {
            phoneNumberView.text = NimUIKit.getContactProvider().getPhoneNumberFrom(contact.phoneNumber)
            contactNameView.text = contact.displayName

            checkView.setOnClickListener {
                item.isChecked = !it.isSelected
                it.isSelected = item.isChecked
                adapter?.onClickCallback(item)
            }

            if (item.contact.isSendInviteSMS) {
                checkView.visibility = View.INVISIBLE
                sendAgainBtn.visibility = View.VISIBLE
            } else {
                checkView.visibility = View.VISIBLE
                sendAgainBtn.visibility = View.INVISIBLE
            }

            val clickEvent = View.OnClickListener { _ ->
                checkView.visibility = View.INVISIBLE
                sendAgainBtn.visibility = View.VISIBLE

                val phoneNumbers = ArrayList<String>()
                phoneNumbers.add(item.number)
                item.contact.setIsSendInviteSMS(true)
                ContactManager.getInstance().contactProvider.changeSendSmsStatus(phoneNumbers)
                val name = contact.displayName
                val number = contact.phoneNumber
                sendInvitation(name, number)

                adapter?.onClickCallback(item)
            }

            checkView.setOnClickListener(clickEvent)
            sendAgainBtn.setOnClickListener(clickEvent)
        }

    }

    override fun inflate(inflater: LayoutInflater): View {
        return inflater.inflate(R.layout.check_contact_view_holder, null)
    }

    @SuppressLint("CheckResult")
    private fun sendInvitation(name: String, number: String) {

        val request = CreateInviteUrlRequest.newBuilder()
                .setInviteeName(name)
                .setInviteeTelephone(number)
                .setSource(InviteSource.SMS)
                .build()

        RPB2CReqSender.createInviteUrl(request)
                .subscribe(Consumer { response ->
                    val body = response.message + response.url
                    sendSms(number, body)
                }, RxGrpcErrorHandler(context))

    }

    private fun sendSms(number: String, body: String) {
        val sendIntent = Intent(Intent.ACTION_SENDTO)
        sendIntent.data = Uri.parse("smsto:$number")
        sendIntent.putExtra("sms_body", body)
        NimUIKit.getContext().startActivity(sendIntent)
    }
}