package com.netease.nim.uikit.business.redpacket.viewholder

import android.graphics.Color
import android.text.Spannable
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.style.BackgroundColorSpan
import android.text.style.ForegroundColorSpan
import android.view.View
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.session.extension.RedPacketOpenedAttachment
import com.netease.nim.uikit.business.session.viewholder.MsgViewHolderBase
import com.netease.nim.uikit.business.team.helper.TeamHelper
import com.netease.nim.uikit.common.ui.recyclerview.adapter.BaseMultiItemFetchLoadAdapter
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.netease.nimlib.sdk.uinfo.model.NimUserInfo

/**
 * 模仿 MsgViewHolderTip
 * 与 MsgViewHolderTip 共用一套布局
 * 但是由于文案处理和事件处理都不一致，
 * 所以单独做成一个holder
 */
class MsgViewHolderOpenRedPacket : MsgViewHolderBase {

    var textView: TextView? = null

    constructor(adapter: BaseMultiItemFetchLoadAdapter<*, *>?) : super(adapter)

    override fun getContentResId(): Int = R.layout.nim_message_item_red_packet_open_tip

    override fun inflateContentView() {
        textView = findViewById(R.id.message_item_notification_label)
    }

    override fun bindContentView() {
        val attachment: RedPacketOpenedAttachment = message.attachment as RedPacketOpenedAttachment
        textView?.text = getOpenRedPacketText(attachment)

        // 保留宽度用于占位
        avatarLeftContainer.layoutParams.height = 1
        avatarRightContainer.layoutParams.height = 1
        avatarLeftContainer.visibility = View.INVISIBLE
        avatarRightContainer.visibility = View.INVISIBLE
        progressBar.visibility = View.GONE


        // 为了和 MsgViewHolderTip 样式一样
        // 所以要设置父容器的 margin，MsgViewHolderTip
        // 在 MsgViewHolderBase 中也做了同样的特殊处理
        (contentContainer.layoutParams as LinearLayout.LayoutParams).setMargins(0, 0, 0, 0)
        (msgRootLayout.layoutParams as FrameLayout.LayoutParams).setMargins(0, 0, 0, 0)
    }

    override fun setStatus() {
        // 不做状态处理
    }

    private fun getOpenRedPacketText(rpoAttachment: RedPacketOpenedAttachment?): SpannableStringBuilder? {
        rpoAttachment ?: return null
        val contentBuilder = SpannableStringBuilder()

        rpoAttachment.apply {
            if (isMine && isOpenMyself) {
                contentBuilder.append(NimUIKit.getContext().getString(R.string.opened_red_packet_yourself))
            } else {
                val openName = rpoAttachment.getOpenName(message.sessionId)
                val senderName = rpoAttachment.getSenderName(message.sessionId)
                contentBuilder.append(context.getString(R.string.someone_opened_red_packet_someone,openName, senderName))
            }

            // "红包" 用红色字色
            contentBuilder.setSpan(ForegroundColorSpan(Color.parseColor("#fff59625")), contentBuilder.length - 2, contentBuilder.length, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)

            // 如果我发送的红包被领完了，那么要加上"，你的红包被领完"
            if (isMineLast) {
                contentBuilder.append(NimUIKit.getContext().getString(R.string.your_red_packet_is_finish))
            }
        }

        return contentBuilder
    }

    override fun isMiddleItem(): Boolean {
        return true
    }

    override fun isShowHeadImage(): Boolean {
        return false
    }
}