package com.netease.nim.uikit.business.redpacket.viewholder

import android.text.TextUtils
import android.view.TextureView
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.session.extension.RedPacketAttachment
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.netease.nimlib.sdk.msg.model.IMMessage

open class RedPacketViewHolderWrapper(private val message: IMMessage,
                                      private val rootView: View) {


    fun getContentResId(): Int = R.layout.nim_message_item_red_packet

    fun bindContentView() {

        val attachment: RedPacketAttachment = message.attachment as RedPacketAttachment
        val contentTextView = rootView.findViewById<TextView>(R.id.txt_card_content)
        val messageTextView = rootView.findViewById<TextView>(R.id.txt_card_title)
        val redPacketTextView = rootView.findViewById<TextView>(R.id.red_packet_text)
        val bmPayIconView = rootView.findViewById<ImageView>(R.id.bm_pay_icon_view)
        val imgCardView = rootView.findViewById<ImageView>(R.id.img_card_avatar)
        messageTextView.text = attachment.message

        if (isGetDone()) {
            rootView.findViewById<View>(R.id.txt_card_content).alpha = 0.4f
            imgCardView.setImageResource(R.drawable.money_open)
            imgCardView.alpha = 0.4f
            bmPayIconView.alpha = 0.4f
            contentTextView.alpha = 0.4f
            messageTextView.alpha = 0.4f
            redPacketTextView.alpha = 0.4f
            contentTextView.setText(R.string.opened_red_packet)
        } else if(isExpired()){
            rootView.findViewById<View>(R.id.txt_card_content).alpha = 0.4f
            imgCardView.setImageResource(R.drawable.money)
            imgCardView.alpha = 0.4f
            bmPayIconView.alpha = 0.4f
            contentTextView.alpha = 0.4f
            messageTextView.alpha = 0.4f
            redPacketTextView.alpha = 0.4f
            contentTextView.setText(R.string.opened_red_packet_is_expired)
        } else if(inFinish()){
            rootView.findViewById<View>(R.id.txt_card_content).alpha = 0.4f
            imgCardView.setImageResource(R.drawable.money)
            imgCardView.alpha = 0.4f
            bmPayIconView.alpha = 0.4f
            contentTextView.alpha = 0.4f
            messageTextView.alpha = 0.4f
            redPacketTextView.alpha = 0.4f
            contentTextView.setText(R.string.opened_red_packet_is_finish)
        }else {
            rootView.findViewById<ImageView>(R.id.img_card_avatar).setImageResource(R.drawable.money)
            rootView.findViewById<View>(R.id.txt_card_title).alpha = 1f
            rootView.findViewById<View>(R.id.txt_card_content).alpha = 1f
            imgCardView.alpha = 1f
            bmPayIconView.alpha = 1f
            contentTextView.alpha = 1f
            messageTextView.alpha = 1f
            redPacketTextView.alpha = 1f

            if (TextUtils.equals(message.fromAccount, NimUIKitImpl.getAccount()) && !isTeamType()) {
                contentTextView.setText(R.string.check_red_packet)
            } else {
                contentTextView.setText(R.string.open_red_packet)
            }
        }
    }

    fun leftBackground(): Int {
        return if (isGetDone()) {
            NimUIKitImpl.getOptions().redPacketDisableLeftBackground
        } else {
            NimUIKitImpl.getOptions().redPacketLeftBackground
        }
    }

    fun rightBackground(): Int {
        return if (isGetDone()) {
            NimUIKitImpl.getOptions().redPacketDisableRightBackground
        } else {
            NimUIKitImpl.getOptions().redPacketRightBackground
        }
    }

    fun isGetDone(): Boolean {
        return (message.attachment as RedPacketAttachment).isGetDone
    }

    fun isExpired(): Boolean{
        return (message.attachment as RedPacketAttachment).isExpired
    }

    fun inFinish(): Boolean {
        return (message.attachment as RedPacketAttachment).isFinish
    }

    fun isTeamType(): Boolean{
        val redPacketAttachment = message.attachment as RedPacketAttachment
        return redPacketAttachment.redPacketType == RedPacketAttachment.RedPacketType.TeamAVG
                || redPacketAttachment.redPacketType == RedPacketAttachment.RedPacketType.TeamLucky
    }
}