package com.netease.nim.uikit.business.session.module.input

import android.content.ClipData
import android.content.ClipDescription
import android.view.DragEvent
import android.view.View
import com.bullet.libcommonutil.log.Logger

/**
 * @author hedongjin <a href="mailto:hedongjin@smartisan.com">Contact me.</a>
 * @version 1.0
 * @since 18-7-30
 */
object DragInputUtils {

    const val TYPE_TEXT = 0L
    const val TYPE_IMAGE = 1L
    const val TYPE_AUDIO = 2L
    const val TYPE_VIDEO = 3L
    const val TYPE_FILE = 4L
    const val TYPE_UNKNOWN = 5L

    fun addCommonProcess(view: View, process: (MutableList<InputData>) -> Unit) {
        view.setOnDragListener { v, event ->
            run {
                Logger.i("DragInputUtils", "${v.javaClass.simpleName} clipDescription = ${event.clipDescription} clipData = ${event.clipData}")
                if (event.action == DragEvent.ACTION_DROP) {
                    process.invoke(parseClipData(event) ?: return@setOnDragListener true)
                }
                true
            }

        }
    }

    private fun parseClipData(event: DragEvent): MutableList<InputData>? {
        val clipData = event.clipData ?: return null
        val clipDesc = event.clipDescription ?: return null
        if (clipData.itemCount != clipDesc.mimeTypeCount || clipData.itemCount <= 0) {
            Logger.i("DragInputUtils", "clipData.itemCount = ${clipData.itemCount} clipDesc.mimeTypeCount = ${clipDesc.mimeTypeCount}")
            return null
        }

        return (0 until clipDesc.mimeTypeCount)
                .map {
                    val mimeItem = clipData.getItemAt(it)
                    val mimeType = clipDesc.getMimeType(it)
                    InputData(when {
                        mimeType.hasMimeTypes("text/*") -> TYPE_TEXT
                        mimeType.hasMimeTypes("image/*") -> TYPE_IMAGE
                        mimeType.hasMimeTypes("audio/*") -> TYPE_AUDIO
                        mimeType.hasMimeTypes("video/*") -> TYPE_VIDEO
                        mimeType.hasMimeTypes("application/*") -> TYPE_FILE
                        else -> TYPE_UNKNOWN
                    }, mimeItem)
                }
                .sortedBy { it.type }
                .toMutableList()
    }

    private fun String.hasMimeTypes(vararg extension: String): Boolean {
        return extension.any {
            ClipDescription.compareMimeTypes(this, it)
        }
    }

    data class InputData(val type: Long, val data: ClipData.Item)
}