package com.netease.nim.uikit.business.websearch

import android.text.TextUtils
import android.text.format.DateUtils
import android.util.Log
import com.bullet.chat.grpc.DeviceType
import com.bullet.chat.grpc.GenerateTempTokenRequest
import com.bullet.chat.grpc.GenerateTempTokenResponse
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog
import com.bullet.messenger.preferences.UserPreferences
import com.netease.nim.uikit.business.websearch.imbrowser.WebViewCompat
import com.netease.nim.uikit.common.presenter.HandleDisposablePresenter
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.grpc.ProtoBuffManager
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import smartisan.cloud.im.rxjava.RxGrpcResultHandler

class WebTokenCache : HandleDisposablePresenter() {

    companion object {
        private val TAG = WebTokenCache::class.java.simpleName
        private const val LAST_TIME_KEY = "LAST_TIME_KEY"
        private const val TOKEN_KEY = "TOKEN_KEY"
        private const val TIME = 5 * DateUtils.MINUTE_IN_MILLIS
    }

    fun getToken(resultCallback: (String?) -> Unit) {
        if(!checkTime()){
            resultCallback.invoke(getTokenFromCache())
            return
        }
        Observable.create(object : RxGrpcResultHandler<GenerateTempTokenResponse>() {
            override fun subscribe(): GenerateTempTokenResponse {
                val request = GenerateTempTokenRequest.newBuilder()
                        .setDeviceType(DeviceType.DEVICE_TYPE_ANDROID)
                        .build()
                return ProtoBuffManager.getInstance()
                        .doGetAccountStub()
                        .generateTempToken(request)
            }
        }).subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer { response ->
                    saveToken(response.accessToken)
                    resultCallback.invoke(response.accessToken)
                    NimLog.i(TAG, "token success!")
                }, RxGrpcErrorHandler())
                ?.let { collectDisposable(it) }
    }

    private fun getLastTime(): Long {
        return UserPreferences.getLong(LAST_TIME_KEY)
    }

    private fun setLastTime(time: Long) {
        UserPreferences.saveLong(LAST_TIME_KEY, time)
    }

    private fun saveToken(token: String) {
        UserPreferences.saveString(TOKEN_KEY, token)
    }

    private fun getTokenFromCache() : String?{
        return UserPreferences.getString(TOKEN_KEY)
    }

    private fun checkTime(): Boolean {
        val currentTime = System.currentTimeMillis()
        if (currentTime - getLastTime() > TIME) {
            setLastTime(currentTime)
            return true
        }
        return false
    }
}