package com.netease.nim.uikit.common.activity

import android.Manifest
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.KeyEvent
import android.view.MotionEvent
import android.widget.Toast
import com.bullet.chat.grpc.ErrorCode
import com.bullet.libcommonutil.log.Logger
import com.bullet.libcommonutil.permission.AutoBootPermissionUtil
import com.bullet.libcommonutil.permission.MPermission
import com.bullet.libcommonutil.permission.annotation.OnMPermissionDenied
import com.bullet.libcommonutil.permission.annotation.OnMPermissionGranted
import com.bullet.libcommonutil.permission.annotation.OnMPermissionNeverAskAgain
import com.bullet.libcommonutil.util.RomUtil
import com.bullet.libcommonutil.util.UIHandler
import com.bullet.messenger.business.base.DataAnalytics
import com.bullet.messenger.business.base.DataTrackEvents
import com.bullet.messenger.contact.cache.FriendDataCache
import com.bullet.messenger.preferences.AppSharedPreferences
import com.bullet.messenger.preferences.UserPreferences
import com.google.protobuf.Empty
import com.netease.nim.avchatkit.activity.AVChatActivity
import com.netease.nim.avchatkit.constant.AVChatExtras
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.api.model.main.LoginSyncDataStatusObserver
import com.netease.nim.uikit.business.contact.fragment.SystemMessageFragment
import com.netease.nim.uikit.business.redpacket.event.B2CTaskRouter
import com.netease.nim.uikit.business.session.activity.BaseVideoActivity
import com.netease.nim.uikit.business.session.audio.MessageAudioControl
import com.netease.nim.uikit.business.session.constant.Extras
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import com.netease.nimlib.sdk.NIMClient
import com.netease.nimlib.sdk.NimIntent
import com.netease.nimlib.sdk.RequestCallback
import com.netease.nimlib.sdk.friend.FriendService
import com.netease.nimlib.sdk.friend.constant.VerifyType
import com.netease.nimlib.sdk.friend.model.AddFriendData
import com.netease.nimlib.sdk.mixpush.MixPushService
import com.netease.nimlib.sdk.msg.MsgService
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import com.smartisan.libstyle.dialog.BulletAlertMessageDialog
import com.smartisan.libstyle.toast.NimToast
import org.json.JSONObject
import smartisan.cloud.im.GrpcCallback
import smartisan.cloud.im.LocalAccountInfoManager
import smartisan.cloud.im.framwork.NimSingleThreadExecutor
import smartisan.cloud.im.grpc.GrpcLoginManager
import smartisan.cloud.im.grpc.GrpcUserInfoManager

/**
 * @author hedongjin <a href="mailto:hedongjin@smartisan.com">Contact me.</a>
 * @version 1.0
 * @since 18-7-31
 */
abstract class MainActivityBase : BaseVideoActivity() {

    private val TAG = this.javaClass.simpleName

    override fun onCreate(savedInstanceState: Bundle?) {
        setSlideable(false)
        super.onCreate(savedInstanceState)
    }

    protected fun handleBaseLogic() {
        observeCacheSync()
        observeLoginSync()
//        checkUnactivedFriend()
        requestPermission()
        checkAutoStartDialog()
        checkAutoAddFriend()

        //初始化语音播放器的听筒模式
        MessageAudioControl.getInstance(this).setEarPhoneModeEnable(UserPreferences.isEarPhoneModeEnable())
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        intent?.let { parseIntent(it) }
    }

    private fun requestPermission() {
        MPermission.printMPermissionResult(true, this, BASIC_PERMISSIONS)
        MPermission.with(this)
                .setRequestCode(BASIC_PERMISSIONS_REQUEST_CODE)
                .permissions(*BASIC_PERMISSIONS)
                .request()
    }

    private fun observeLoginSync() {
        if (!LoginSyncDataStatusObserver.getInstance().observeSyncDataCompletedEvent {
            notifyLoginSyncCompleted()
            DialogMaker.dismissProgressDialog()
        }) {
            DialogMaker.showProgressDialog(this, getString(R.string.prepare_data)).setCanceledOnTouchOutside(false)
        } else if (!TextUtils.isEmpty(NimUIKit.getAccount())) {
          notifyLoginSyncCompleted()
        }
    }

    open fun observeCacheSync() {
        if (!NimUIKit.isInitComplete()) {
            val runnable = Runnable {
                NimUIKit.setInitStateListener(null)
                notifyCacheSyncCompleted()
            }

            NimUIKit.setInitStateListener {
                UIHandler.removeCallbacks(runnable)
                UIHandler.post(runnable)
            }

            UIHandler.postDelayed(runnable, 3000)
        } else {
            notifyCacheSyncCompleted()
        }
    }

    open fun notifyLoginSyncCompleted() {}

    open fun notifyCacheSyncCompleted() {
        parseIntent(intent)
    }

    protected fun parseIntent(intent: Intent) {
        when {
            TextUtils.isEmpty(NimUIKit.getAccount()) -> handleLogin()
            AppSharedPreferences.shouldLogoutToEnter() -> handleLogout()
            NIMClient.getService(MixPushService::class.java).isFCMIntent(intent) -> handleFCMIntent(intent)
            intent.hasExtra(NimIntent.EXTRA_NOTIFY_CONTENT) -> handleNotifyIntent(intent)
            intent.hasExtra(SystemMessageFragment.EXTRA_FROM_NEW_FRIEND) -> handleSystemMessageIntent(intent)
            intent.hasExtra(Extras.EXTRA_FROM_OMA_NOTIFICATION) -> handleOMAIntent(intent)
            intent.hasExtra(AVChatExtras.EXTRA_FROM_NOTIFICATION) -> handleAVChatNotifyIntent(intent)
            intent.hasExtra(AVChatActivity.INTENT_ACTION_AVCHAT) -> handleAVChatActionIntent(intent)
            intent.hasExtra(EXTRA_APP_QUIT) -> handleQuit(intent)
            intent.hasExtra(EXTRA_APP_KO) -> handleKo(intent)
            intent.hasExtra(B2CTaskRouter.IS_B2C_TASK_KEY) -> handlerB2CTaskRouter(intent)
        }
    }

    abstract fun handleLogin()

    abstract fun handleLogout()

    abstract fun handleFCMIntent(intent: Intent)

    abstract fun handleNotifyIntent(intent: Intent)

    abstract fun handleSystemMessageIntent(intent: Intent)

    abstract fun handleAVChatNotifyIntent(intent: Intent)

    abstract fun handleAVChatActionIntent(intent: Intent)

    abstract fun handleKo(intent: Intent)

    abstract fun handleOMAIntent(intent: Intent)
    open fun handlerB2CTaskRouter(intent: Intent){}

    // 注销
    open fun handleQuit(intent: Intent) {
        DialogMaker.showProgressDialog(this, "")
        GrpcLoginManager.getInstance().logout(object : GrpcCallback<Empty>() {
            override fun onSuccess(empty: Empty) {
                Logger.grpc("onLogout onSuccess----")
                DialogMaker.dismissProgressDialog()
                handleKo(intent)
            }

            override fun onFailed(code: Int, errorMsg: String) {
                DialogMaker.dismissProgressDialog()
                Logger.grpc("onLogout onFailed----:$errorMsg, code:$code")
                if (code == ErrorCode.ERROR_UNAUTHENTICATED_VALUE) {
                    handleKo(intent)
                } else {
                    NimToast.makeToast(applicationContext, errorMsg, Toast.LENGTH_SHORT).show()
                }
            }
        })
    }


    /**
     * 登录时把recentcontact里不是好友的用户加为好友
     * 向非子弹短信用户发送子弹短信，邀请注册方登录时需建立好友关系
     */
    private fun checkUnactivedFriend() {
        if (TextUtils.isEmpty(NimUIKit.getAccount())) return
        NimSingleThreadExecutor.getInstance().execute {
            val recentContacts = NIMClient.getService(MsgService::class.java).queryRecentContactsBlock() ?: return@execute

            recentContacts.filter {
                it.sessionType == SessionTypeEnum.P2P
                        && !FriendDataCache.getInstance().isMyFriend(it.contactId)
                        && it.unreadCount > 0
            }.forEach {
                Logger.d(TAG, "not friend & unread recentcontact accountid=" + it.contactId + ";name=" + it.fromNick)

                NIMClient.getService(FriendService::class.java).addFriend(AddFriendData(it.contactId, VerifyType.DIRECT_ADD, null))
                        .setCallback(object : RequestCallback<Void> {
                            override fun onSuccess(param: Void?) {
                                Logger.d(TAG, "verify add friend success")
                            }

                            override fun onFailed(code: Int) {
                                Logger.d(TAG, "add friend failed code =" + code)
                            }

                            override fun onException(exception: Throwable?) {
                                Logger.d(TAG, "add friend onException:" + exception)
                            }
                        })
            }
        }
    }

    private var mAutoStartTipDialog: BulletAlertMessageDialog? = null
    private fun checkAutoStartDialog() {
        if (!UserPreferences.getFirstLogin()) return
        UserPreferences.saveFirstLogin(false)

        if (!RomUtil.isSmartisan()) {
            val builder:BulletAlertMessageDialog.Builder = BulletAlertMessageDialog.Builder(this);
            mAutoStartTipDialog = builder.setTitle(R.string.auto_start_dialog_title)
                    .setMessage(R.string.auto_start_dialog_message)
                    .hideNegativeButton(true)
                    .setPositiveButton(R.string.auto_start_dialog_button) { dialogInterface: DialogInterface, _: Int ->
                        AutoBootPermissionUtil.startDeviceAutoBootActivity(this)
                        dialogInterface.dismiss()
                        sendEvent(resources.getString(R.string.auto_start_dialog_title))
                    }
                    .setOnCancelListener {
                        sendEvent("x")
                    }
                    .setCancelable(false)
                    .create()
            mAutoStartTipDialog!!.show()
        }

    }

    private fun checkAutoAddFriend() {
        if (TextUtils.isEmpty(NimUIKit.getAccount())) return
        val userInfo = LocalAccountInfoManager.getInstance().loginUser
        if (!userInfo.isSetedAutoAddFriend) {
            val dialog = BulletAlertMessageDialog.Builder(this)
                    .setTitle(R.string.auto_add_friend_title)
                    .setMessage(R.string.auto_add_friend_message)
                    .setNegativeButton(R.string.not_now, { _, _ ->
                        updateAutoAddFriendSetting(false)
                        sendDataAnalytics("Click_Cancel")
                    })
                    .setPositiveButton(R.string.ok, { _, _ ->
                        updateAutoAddFriendSetting(true)
                        sendDataAnalytics("Click_OK")
                    })
                    .setOnKeyListener({ _, keyCode, event ->
                        if (keyCode == KeyEvent.KEYCODE_BACK && event.repeatCount == 0 && event.action == MotionEvent.ACTION_DOWN) {
                            sendDataAnalytics("Android_Physical_ReturnButton")
                        }
                        false
                    })
                    .hideCloseButton(true)
                    .create()
            dialog.show()
        }
    }

    private fun sendDataAnalytics(param: String) {
        DataAnalytics.getInstance().start(DataTrackEvents.AUTOADDFRIEND_POPUP)
                .addValue("Click_OK", "Click_OK" == param)
                .addValue("Click_Cancel", "Click_Cancel" == param)
                .addValue("Android_Physical_ReturnButton", "Android_Physical_ReturnButton" == param)
                .done()
    }

    private fun updateAutoAddFriendSetting(autoAdd: Boolean) {
        GrpcUserInfoManager.getInstance().editAutoAddFriendSetting(autoAdd, GrpcCallback())
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        MPermission.onRequestPermissionsResult(this, requestCode, permissions, grantResults)

        supportFragmentManager.fragments?.forEach {
            it.onRequestPermissionsResult(requestCode, permissions, grantResults)
        }
    }

    @OnMPermissionGranted(BASIC_PERMISSIONS_REQUEST_CODE)
    fun onBasicPermissionSuccess() {
        MPermission.printMPermissionResult(false, this, BASIC_PERMISSIONS)
    }

    @OnMPermissionDenied(BASIC_PERMISSIONS_REQUEST_CODE)
    @OnMPermissionNeverAskAgain(BASIC_PERMISSIONS_REQUEST_CODE)
    fun onBasicPermissionFailed() {
        NimToast.makeToast(this, "未全部授权，部分功能可能无法正常运行！", Toast.LENGTH_SHORT).show()
        MPermission.printMPermissionResult(false, this, BASIC_PERMISSIONS)
    }

    companion object {
        const val EXTRA_APP_QUIT = "APP_QUIT"
        const val EXTRA_APP_KO = "APP_KO"
        const val BASIC_PERMISSIONS_REQUEST_CODE = 100
        val BASIC_PERMISSIONS = arrayOf(
                Manifest.permission.READ_PHONE_STATE,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.READ_EXTERNAL_STORAGE,
                Manifest.permission.CAMERA,
                Manifest.permission.RECORD_AUDIO,
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.READ_CALL_LOG)

    }

    private fun sendEvent(btnName: String){
        var json = JSONObject()
                .put(DataTrackEvents.POPUP_TITLE, btnName)
                .put(DataTrackEvents.POPUP_AT_PAGE, "列表页")
                .put(DataTrackEvents.BUTTON_NAME, resources.getString(R.string.auto_start_dialog_button))
        DataAnalytics.getInstance().track(DataTrackEvents.AUTHO_AUTOSTART_POPUP, json)
    }

}