package com.netease.nim.uikit.common.keyboard

import android.content.res.Configuration
import android.view.KeyEvent
import android.view.View
import com.netease.nim.uikit.api.NimUIKit
import com.bullet.libcommonutil.log.Logger

/**
 * @author hedongjin <a href="mailto:hedongjin@smartisan.com">Contact me.</a>
 * @version 1.0
 * @since 18-7-13
 */
class KeyboardContext(size: Int = 20) : View.OnKeyListener {

    private val history = LimitLinkedList<Int>(size)
    private val process = HashMap<Array<Int>, IKeyboardProcess>()

    override fun onKey(v: View, keyCode: Int, event: KeyEvent): Boolean {
        try {
            if (NimUIKit.getContext().resources.configuration.hardKeyboardHidden != Configuration.HARDKEYBOARDHIDDEN_NO) {
                return false
            }

            if (!containsKeyCode(keyCode)) {
                return false
            } else {

                if (event.action == KeyEvent.ACTION_UP && history.contains(keyCode)) {
                    history.remove(keyCode)
                }

                if (event.action == KeyEvent.ACTION_DOWN && !history.contains(keyCode)) {
                    history.add(keyCode)
                }

                if (event.action == KeyEvent.ACTION_DOWN && history.size > 0) {
                    return getKeyboardProcess(historyArray())?.process() ?: return true
                }

                return true
            }
        } finally {
            Logger.i("KeyboardContext", "action = ${event.action} keyCode = $keyCode history = $history")
        }
    }

    private fun historyArray(): Array<Int> {
        val array = arrayOfNulls<Int>(history.size)
        return history.toArray(array)
    }

    private fun getKeyboardProcess(key: Array<Int>): IKeyboardProcess? {
        for (entry in process) {
            if (equalsArray(key, entry.key)) {
                return entry.value
            }
        }
        return null
    }

    private fun equalsArray(a1: Array<Int>, a2: Array<Int>): Boolean {
        if (a1.size != a2.size) return false
        return a1.any { it -> a2.contains(it) }

    }

    private fun containsKeyCode(keyCode: Int): Boolean {
        return process.keys.any { it.contains(keyCode) }
    }

    fun addKeyboardProcess(key: Array<Int>, process: IKeyboardProcess) {
        this.process.put(key, process)
    }


    interface IKeyboardProcess {
        fun process(): Boolean
    }


}