package com.netease.nim.uikit.common.ui.recyclerview

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.util.AttributeSet
import android.view.View

/**
 * @author hedongjin <a href="mailto:hedongjin@smartisan.com">Contact me.</a>
 * @version 1.0
 * @since 18-7-25
 */
class EmptyRecyclerView(context: Context, attributeSet: AttributeSet?) : RecyclerView(context, attributeSet) {

    var emptyView: View? = null

    val observer = object : AdapterDataObserver() {
        override fun onChanged() {
            super.onChanged()
            checkEmpty()
        }

        override fun onItemRangeRemoved(positionStart: Int, itemCount: Int) {
            super.onItemRangeRemoved(positionStart, itemCount)
            checkEmpty()
        }

        override fun onItemRangeInserted(positionStart: Int, itemCount: Int) {
            super.onItemRangeInserted(positionStart, itemCount)
            checkEmpty()
        }
    }

    override fun setAdapter(adapter: Adapter<*>?) {
        getAdapter()?.unregisterAdapterDataObserver(observer)
        super.setAdapter(adapter)
        adapter?.registerAdapterDataObserver(observer)
        checkEmpty()
    }

    private fun checkEmpty() {
        val emptyNotNull = emptyView ?: return
        val adapterNotNull = adapter ?: return

        if (adapterNotNull.itemCount == 0) {
            emptyNotNull.visibility = View.VISIBLE
            this.visibility = View.GONE
        } else {
            emptyNotNull.visibility = View.GONE
            this.visibility = View.VISIBLE
        }
    }
}